"""
The core library for Icosahedral reconstruction.
Coded by Wen Jiang (wjiang@bcm.tmc.edu), May 2001
"""

import sys, os, os.path, fileinput, copy, math, string, types, re, struct

# make sure Python version is recent ( >= V2.0 ) 
if sys.hexversion < 33554673: raise ValueError("SAVR requires Python version 2.0 and above to run. You are running \"%s\" of version \"%s\" " % (sys.executable, sys.version))

def SendMail(mailserver="localhost",to="",subject="SAVR Automated Processing Progress Report",msg="",images=[]):
	""" Send E-Mail to inform the processing progress
	"""	
	import smtplib, socket, StringIO, MimeWriter, base64, getpass
	fromaddr=getpass.getuser()+"@"+socket.gethostname()
	if not to: 
		to=fromaddr
		tos=[[to,'']]
	else:
		mailaddrp=re.compile(r"([A-Za-z0-9._-]+@[A-Za-z0-9_-]+(\.[A-Za-z0-9_-]+)*)")
		tos=mailaddrp.findall(to)

	fullto=""
	for cto in tos: fullto=fullto+cto[0]+" "

	outputfp = StringIO.StringIO()
	
	# Create a MimeWriter object
	
	w = MimeWriter.MimeWriter(outputfp)
	w.addheader("Subject", subject)
	w.addheader("From",fromaddr)
	w.addheader("To", fullto)
	w.addheader("MIME-Version", "1.0")
	w.flushheaders()
	
	# Now we create a plain text segment and write the instructions file into it
	
	w.startmultipartbody("mixed")
	instructions = w.nextpart()
	instFile = instructions.startbody("text/plain")
	instructions.flushheaders()
	instFile.write(msg)
	
	# Now we create a base64 encoded segment and write the diary file into it as inline images
	if images:
		for img in images:
			subwriter = w.nextpart()
			subwriter.addheader("Content-Transfer-Encoding", "base64")
			subwriter.addheader("Content-Disposition", 'inline; filename="%s"' % img)
			#f = subwriter.startbody('application/octet-stream; name="%s"' % img)
			f = subwriter.startbody('image/jpeg; name="%s"' % img)
			subwriter.flushheaders()
			f.write(base64.encodestring(open(img, 'rb').read()))
	w.lastpart()

	try:
		server=os.popen("sendmail %s" % fullto, "w")
		server.write(outputfp.getvalue())
		server.close()
	except:
		server = smtplib.SMTP(mailserver)
		server.set_debuglevel(0)
		server.sendmail(fromaddr, fullto, outputfp.getvalue())
		server.close()                                    
def SendUsage():
	"""Send short log message to server to record the usage of SAVR software"""
	return 
	import socket, getpass, time
	port = 8081
	host = "128.249.148.233"
	s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
	s.bind(("", 0))
	data={"software":"SAVR","user":getpass.getuser(), "client":socket.gethostname(), "time":time.ctime()}
	s.sendto("%s" % (data), (host, port))
def NumberOfIcosProjections(step):
	""" Compute number of sampling points in the Icosahedral Asymmetric Triangle for the specified angular step.
		It uses the same sampling strategy as used by Hong
	"""
	ALT0=69.0948425521
	ANG5to5HLF=31.7174744115  
	count=0
	alt = ALT0
	while alt < 90.+step/100.:
		az=step
		count=count+1
		while az<=ANG5to5HLF*(alt-ALT0) / (90.-ALT0):
			count=count+2
			az=az+step
		alt=alt+step 
	return count
def CTFCurve(defocus,B=0,ampcontrast=0.1,voltage=400,Cs=4.1,apix=2.7,noise1=0,noise2=0,noise3=0,noise4=0,imagesize=256,res=5,mode=0):	
	""" Compute the CTF curve. defocus should be negative for underdefocus
		mode:	0: CTF amplitude
				1: CTF*CTF intensity
				2: 1/[CTF*(1+(Noise/Signal)^2] wiener filter
	"""
	if defocus>0: print "WARNING: the defocus should be negative for under-defocus"
	invstep=1./(imagesize*apix)
	invres=1./res
	s=0	
	i=0
	ret=[]
	while s <= invres:
		wl=12.2639/math.sqrt(voltage*1000.0+.97845*voltage*voltage)
		#print "Cs ", Cs, " defocus ", defocus, " Wavelength ", wl, " frequency ", s
		gamma=2*math.pi*(2.5e6*Cs*wl*wl*wl*s*s*s*s+5000.*defocus*wl*s*s)
		ctf=math.exp(-B*s*s)*(math.sqrt(1-ampcontrast*ampcontrast)*math.sin(gamma)-ampcontrast*math.cos(gamma))
		if mode==0:	 ret.append([s,ctf])
		elif mode==1: ret.append([s,ctf*ctf])
		elif mode==2: ret.append([s,1./(ctf*(1.+noise1*math.exp(-noise2*s-noise3*s*s-noise4*math.sqrt(s))))])	
		i=i+1
		s=invstep*i
	return ret	
def OrtAll(imgfile, apix,startimg=0, mask1=0, mask2=0, res1=3, res2=30,verbose=0):
	"""	Run ortAll (a program that finds the center and initial orientation using self common line algorithm).
		It accepts an IMAGIC image file as input and splits into individual MRC images to feed ortAll
	"""
	info=ImageInfo(imgfile)
	if not mask1: mask1=info["nx"]/2
	if not mask2: mask2=info["nx"]/2
	imgnum=info["num"]
	cmd="proc2d "+imgfile+" mrc mrc invert first="+`startimg`
	if verbose: print cmd
	os.system(cmd)
	cmd="ortAll %d.mrc %d.mrc" % (100+startimg,100+imgnum-1)
	if verbose: 
		print "\n"+cmd
		print "mask1: %d\tmask2: %d\tres1: %d\tres2: %d" %(mask1,mask2,res1,res2)
	fp=os.popen(cmd,'w')
	fp.write("%d\n%d\n" % (mask1,mask2))
	fp.write("%f\n" % apix)
	fp.write("%d %d\n" % (int(res1),int(res2)))
	fp.close()
	return 1
def BuildTemplate(imgfile,height=4.,width=12,angdiff=3,phase=60,res1=3,res2=30,min_num_particles=3,relax=0,verbose=0):
	"""	Find good particles with consistent initial orientations processed by ortAll and run buildTemplate
	"""
	def goodlog(logfile,height=4.,width=12,angdiff=3,phase=60):
		try:
			fp=open(logfile,'r')
		except:
			print "Warning: cannot open log file \"%s\" for initial orientation determination, ignored" % (logfile)
			return None
		lines=fp.readlines()
		h=string.atof(string.split(lines[2],":",2)[1])
		w=string.atof(string.split(lines[3],":",2)[1])
		if h<height or w>width: return None
		
		fields=string.split(lines[2])
		alt=string.atof(fields[0])
		az=string.atof(fields[1])
		phi=string.atof(fields[2])
		fields=string.split(lines[3])
		alt2=string.atof(fields[0])
		az2=string.atof(fields[1])
		phi2=string.atof(fields[2])
		if math.fabs(alt-alt2)>angdiff or math.fabs(az-az2)>angdiff or math.fabs(phi-phi2)>angdiff: return None
		alt=(alt+alt2)/2.
		az=(az+az2)/2.
		phi=(phi+phi2)/2.
		
		hit=0
		for l in range(4,len(lines)):
			fields=string.split(lines[l])
			alt2=string.atof(fields[0])
			az2=string.atof(fields[1])
			phi2=string.atof(fields[2])
			p=string.atof(fields[-1])
			if p<phase and math.fabs(alt-alt2)<=angdiff and math.fabs(az-az2)<=angdiff and math.fabs(phi-phi2)<=angdiff: 
				hit=1
				break
		if hit:	# use the distance from the 4 symmetry axes as the criterion for good candidate. the far the better
			ALT0=69.0948425521
			ANG5to5HLF=31.7174744115  
			axes=[[ALT0,0],[90,-ANG5to5HLF],[90,0],[90,ANG5to5HLF]]
			max=0
			for c in axes:
				dist=(c[0]-alt)*(c[0]-alt)+(c[-1]-az)*(c[-1]-az)
				if dist>max: max=dist
			return logfile, max
		else: return None

	info=ImageInfo(imgfile)
	imgnum=info["num"]
	rank=[]
	if not relax:
		rank=[]
		for i in range(imgnum):
			res=goodlog(`i+100`+'.log',height=height,width=width,angdiff=angdiff,phase=phase)
			if res: rank.append(res)
		if len(rank)>=min_num_particles:
			#rank.sort(lambda a,b:-1*(a[1]>b[1]))
			rank.sort(lambda a,b:cmp(b[1],a[1]))
		else:
			print "Cannot find enough number (more than 3) of good particles to build template, please make sure \"%s\" is a good micrograph" % (imgfile)
			return 0
	else:
		for inc in range(100,relax*100,10):
			rank=[]
			for i in range(imgnum):
				res=goodlog(`i+100`+'.log',height=height/(inc/100.),width=width*inc/100.,angdiff=angdiff*inc/100.,phase=phase*inc/100.)
				if res: rank.append(res)
			if verbose: print "Cylce of relax level: %d%%: found %d good particles" % (inc-100, len(rank))
			if len(rank)>=min_num_particles:
				#rank.sort(lambda a,b:-1*(a[1]>b[1]))
				rank.sort(lambda a,b:cmp(b[1],a[1]))
				break
		if len(rank)<min_num_particles:
			print "Cannot find enough number (more than %d) of good particles to build template, please make sure \"%s\" is a good micrograph" % (min_num_particles,imgfile)
			return 0
			
		
	if len(rank)>3:	lognum=3
	else: lognum = len(rank)
	
	cmd="buildTemplate "
	for i in range(lognum): cmd=cmd+rank[i][0]+" "
	cmd=cmd+"template.log"
	if verbose: 
		print "\n"+cmd

	fp=os.popen(cmd,'w')
	fp.write("%d\n" % (info["nx"]))
	fp.write("%d %d\n" % (res1,res2))
	fp.close()
	return lognum
def EliminateOrt(imgfile, startimg=0, res1=3, res2=30, phase=75, verbose=0):
	"""	Run eliminateOrt program to eliminate the particles with incorrect initial orientation assignments by ortAll.
		eliminateOrt uses cross-common line algorithm.
	"""
	fpr=open("template.log","r")
	lines=fpr.readlines()
	fpr.close()
	linenum=range(len(lines))
	linenum.reverse()
	startline=0
	pattern=re.compile(r"^.*-+$")
	for i in linenum:
		if pattern.search(lines[i]): 
			startline=i+1 
			break
	if not startline: return 0
	fpw=open("template.dat","w")
	fpw.writelines(lines[startline:])
	fpw.close()
	
	info=ImageInfo(imgfile)
	imgnum=info["num"]
	cmd="eliminateort template.dat "+`startimg+100`+".log "+`100+imgnum-1`+".log"
	if verbose: 
		print "\n"+cmd
		print "image size: %d\tres1: %f\tres2: %f" % (info["nx"],res1,res2)
	fp=os.popen(cmd,'w')
	fp.write("%f\n%d\n" % (phase,info["nx"]))
	fp.write("%f %f\n" % (res1,res2))
	fp.close()

	fp=open("definedort.dat","r")
	newgoodimgs=fp.readlines()
	fp.close()
	ptlcnum=len(newgoodimgs)/2
	return ptlcnum
def GlobalRefine(imgfile, step, res1=3, res2=30, cpu=1, verbose=0):
	"""	Run globalRefine program to improve the center and orientation parameters.
		globalRefine uses cross-common line algorithm.
	"""	
	fp=open("definedort.dat")
	ptlcnum=len(fp.readlines())/2
	fp.close()
	cpu2=ptlcnum/3
	if cpu2==0: cpu=1
	elif cpu2<cpu: cpu=cpu2	
	if os.path.exists("globalRefined.dat"): os.remove("globalRefined.dat")
	cmd="globalRefine definedort.dat globalRefined.dat"
	if verbose: 
		print "\n"+cmd
		print "Particle number: %d\tCPU: %d\tstep: %f\tres1: %f\tres2: %f\n" % (ptlcnum, cpu, step, res1, res2)
	fp=os.popen(cmd,'w')
	fp.write("%f %f\n\n\n" % (res1,res2))
	fp.write("%f\n%d\n" % (step, cpu))
	fp.close()
	return 1
def Reconstruct(datfile, diameter, res=30, memo="icos-map", cpu1=1, cpu2=2, verbose=0):
	"""	Run reconstructParellel program to build the 3D model from individual images
		reconstructParellel uses Fourier-Bessel synthesis and MPI
	"""
	fp=open(datfile)
	ptlcnum=len(fp.readlines())/2
	fp.close()
	cmd="reconstruct.py %s --script=All-particles-reconstruct.script --diameter=%d --res=%d --cpu1=%d --cpu2=%d --memo=%s" % (datfile,diameter,res,cpu1,cpu2,memo)
	if verbose:
		print "\n"+cmd
	os.system(cmd)
	return 1
def ImageInfo(filename):
	"""detect the image format, size, and number of images"""
	def CheckAsMRC(file, maxsize=100000):	# check for MRC format
		MRC_FORMAT="10i6f3i3f31i2fi800s"	# the format string to use by struct module for MRC file header
		MRC_MODE={	0:("MODE_char",1),
					1:("MODE_short",2),
					2:("MODE_float",4),
					3:("MODE_short_COMPLEX",4),
					4:("MODE_float_COMPLEX",8)}
		fp=open(file,"rb")		
		imgh=fp.read(struct.calcsize(MRC_FORMAT))
		fp.close()
		mrch=struct.unpack('<'+MRC_FORMAT,imgh)	# check as little endian
		if mrch[3] not in MRC_MODE.keys() or mrch[0]<0 or mrch[0]>maxsize or mrch[1]<0 or mrch[1]>maxsize or mrch[2]<0 or mrch[2]>maxsize:
			mrch=struct.unpack('>'+MRC_FORMAT,imgh)	# now check as big endian
		#print mrch[3], mrch[0], mrch[1], mrch[2]
		if mrch[3] in MRC_MODE.keys() and mrch[0]>0 and mrch[0]<maxsize and mrch[1]>0 and mrch[1]<maxsize and mrch[2]>0 and mrch[2]<maxsize and os.path.getsize(file)==struct.calcsize(MRC_FORMAT)+mrch[0]*mrch[1]*mrch[2]*MRC_MODE[mrch[3]][1]+mrch[23]:
			return {"format":"MRC","mode":MRC_MODE[mrch[3]][0],"nx":mrch[0],"ny":mrch[1],"nz":mrch[2],"num":1}
		else:
			return None
	def CheckAsPIF(file, maxsize=100000):	# check for PIF format
		PIF_FORMAT="2i16s2i32s5i107i"	# the format string to use by struct module for PIF file header
		PIF_MODE={	0:("PIF_CHAR",1),
					1:("PIF_SHORT_INT",2),
					2:("PIF_FLOAT_INT",4),
					3:("PIF_SHORT_INT_COMPLEX",2),
					4:("PIF_FLOAT_INT_COMPLEX",4),
					6:("PIF_BOXED_INT",1),
					7:("PIF_SHORT_FLOAT",2),
					8:("PIF_SHORT_FLOAT_COMPLEX",2),
					9:("PIF_FLOAT",4),
					10:("PIF_FLOAT_COMPLEX",4),
					20:("PIF_MAP_SHORT_INT",2),
					21:("PIF_MAP_FLOAT_INT",4)}

		fp=open(file,"rb")
		imgh=fp.read(struct.calcsize(PIF_FORMAT))
		fp.close()
		pifh=struct.unpack('<'+PIF_FORMAT,imgh)	# check as little endian
		if not(pifh[4] in [0,1] and pifh[10] in PIF_MODE.keys()) or pifh[7]<0 or pifh[7]>maxsize or pifh[8]<0 or pifh[8]>maxsize:
			pifh=struct.unpack('>'+PIF_FORMAT,imgh)	# now check as big endian
		if pifh[4] in [0,1] and pifh[10] in PIF_MODE.keys() and pifh[7]>0 and pifh[7]<maxsize and pifh[8]>0 and pifh[8]<maxsize and os.path.getsize(file)==512+(pifh[7]*pifh[8]*PIF_MODE[pifh[10]][1]+512)*pifh[3]:
			if pifh[3]>1: nz=1
			else: nz=pifh[9]
			return {"format":"PIF","mode":PIF_MODE[pifh[10]][0],"nx":pifh[7],"ny":pifh[8],"nz":nz,"num":pifh[3]}
		else:
			return None
	def CheckAsICOS(file, maxsize=100000):	# check for ICOS format
		ICOS_FORMAT="i72s5i2fi"	# the format string to use by struct module for ICOS file header
		fp=open(file,"rb")		
		imgh=fp.read(struct.calcsize(ICOS_FORMAT))
		fp.close()
		icosh=struct.unpack('<'+ICOS_FORMAT,imgh)	# check as little endian
		if not(icosh[0]==icosh[2] and icosh[0]==72 and icosh[3]==icosh[9] and icosh[3]==20) or icosh[4]<0 or icosh[4]>maxsize or icosh[5]<0 or icosh[5]>maxsize or icosh[6]<0 or icosh[6]>maxsize:
			icosh=struct.unpack('>'+ICOS_FORMAT,imgh)	# now check as big endian
		if icosh[0]==icosh[2] and icosh[0]==72 and icosh[3]==icosh[9] and icosh[3]==20 and icosh[4]>0 and icosh[4]<maxsize and icosh[5]>0 and icosh[5]<maxsize and icosh[6]>0 and icosh[6]<maxsize:
			#and os.path.getsize(file)==struct.calcsize(ICOS_FORMAT)+(icosh[4]+2*4)*icosh[5]*icosh[6]*4:
			return {"format":"ICOS","mode":"FORTRAN__Float","nx":icosh[4],"ny":icosh[5],"nz":icosh[6],"num":1}
		else:
			return None
	def CheckAsIMAGIC(file, maxsize=100000):	# check as IMAGIC format
		hedfile=os.path.splitext(file)[0]+".hed"
		imgfile=os.path.splitext(file)[0]+".img"
		if file[-4:] in [".hed",".img"] and os.path.exists(hedfile) and os.path.exists(imgfile):
			IMAGIC_FORMAT="14i4s2i6f1i5f80s8i4f195i"	# the format string to use by struct module for IMAGIC file header
			IMAGIC_MODE={"REAL":4, "INTG":4}
			fp=open(hedfile,"rb")
			imgh=fp.read(struct.calcsize(IMAGIC_FORMAT))
			fp.close()
			imagich=struct.unpack('<'+IMAGIC_FORMAT,imgh)	# check as little endian
			print file,imagich[0], imagich[1],imagich[4],imagich[5],imagich[13],imagich[12]
			if imagich[1]<0 or imagich[4]>31 or imagich[4]<0 or imagich[5]>12 or imagich[5]<0 or imagich[13]<0 or imagich[13]>maxsize or imagich[12]<0 or imagich[12]>maxsize:	# imgnum, day, month
				imagich=struct.unpack('>'+IMAGIC_FORMAT,imgh)	# now check as big endian
			print file,imagich[0], imagich[1],imagich[4],imagich[5],imagich[13],imagich[12]
			if imagich[1]>=0 and imagich[4]<=31 and imagich[4]>=0 and imagich[5]>=0 and imagich[5]<=12 and imagich[13]>0 and imagich[13]<maxsize and imagich[12]>0 and imagich[12]<maxsize and os.path.getsize(imgfile)==(imagich[1]+1)*imagich[12]*imagich[13]*IMAGIC_MODE[imagich[14]] \
			   and os.path.getsize(hedfile)==(imagich[1]+1)*struct.calcsize(IMAGIC_FORMAT):
				return {"format":"IMAGIC","mode":imagich[14],"nx":imagich[13],"ny":imagich[12],"nz":1, "num":imagich[1]+1}
		return None
	if not os.path.exists(filename): raise IOError("file \"%s\" does not exist" % filename)
	extension=os.path.splitext(filename)[-1]
	if string.lower(extension)==".map": checkorder=[CheckAsICOS,CheckAsMRC,CheckAsIMAGIC,CheckAsPIF]
	elif string.lower(extension) in [".img",".hed"]: checkorder=[CheckAsIMAGIC,CheckAsMRC,CheckAsICOS,CheckAsPIF]
	elif string.lower(extension)==".pif": checkorder=[CheckAsPIF,CheckAsIMAGIC,CheckAsMRC,CheckAsICOS]
	else: checkorder=[CheckAsMRC,CheckAsIMAGIC,CheckAsICOS,CheckAsPIF]
	maxsize = 100000	# maximal size for an image for determinine the file type
	for format in checkorder:
		info=format(filename,maxsize=maxsize)
		if info: break
	if info: return info
	else: raise TypeError("unsupported image file format for \"%s\"" % (filename))
def AutoBoxing(micrograph_file,particle_file,box_file,radius,imagesize,protein_is_white=0,verbose=0,cmdonly=0):
	"""Automatic particle selection using ethan"""
	if imagesize<=radius and imagesize%2: raise ValueError("\"imagesize=%d\" should be larger than \"radius=%d\" and must be even" % (imagesize,radius))
	if not os.path.exists(micrograph_file): raise IOError("micrograph file \"%s\" does not exists" % micrograph_file)
	tmpfname=os.path.splitext(micrograph_file)[0]
	ethanbox=tmpfname+"-ethan.box"
	cmd="cd %s" % (os.path.dirname(os.path.abspath(particle_file)))
	info=ImageInfo(micrograph_file)
	#print micrograph_file, info
	if info["format"] == "MRC" and info["mode"] == "MODE_char":
		micrograph=micrograph_file
	else:
		micrograph=os.path.join(os.path.dirname(particle_file),os.path.splitext(os.path.basename(micrograph_file))[0]+"-mrc8bit.mrc")
		cmd=cmd+" ; proc2d %s %s mrc8bit" % (micrograph_file, micrograph)
	cmd=cmd+"; ethan %d %s %s" % (radius, micrograph, ethanbox)
	if protein_is_white: cmd=cmd+" invert"
	cmd=cmd+"; ethan2boxer.py %s %d %s ; rm %s" % (ethanbox, imagesize, box_file, ethanbox)
	if os.path.exists(particle_file[:-4]+".img"): os.remove(particle_file[:-4]+".img")
	if os.path.exists(particle_file[:-4]+".hed"): os.remove(particle_file[:-4]+".hed")
	cmd=cmd+" ; batchboxer dbbox=%s input=%s output=%s" % (box_file,micrograph_file,particle_file)
	if not protein_is_white: cmd=cmd+" invert"
	if 	micrograph!=micrograph_file: cmd=cmd+"; rm %s" % (micrograph)
	if not cmdonly:
		try:
			if verbose: print cmd
			os.system(cmd)
		except:
			raise RuntimeError("during running command \"%s\"" % (cmd))
		return None
	else:
		return cmd
def RunPar(program, workdir=None, cpus=1, verbose=0):
	"""Use RunPar to run given program in parallel"""
	import tempfile
	if workdir and os.path.exists(workdir): tempfile.tempdir = workdir
	jobfile=tempfile.mktemp(".runpar")
	jobfp=open(jobfile,"w")
	for c in range(cpus):
		jobfp.write("%s frac=%d/%d\n" % (program, c, cpus))
	jobfp.close()
	cmd="runpar proc=%d,%d file=%s" % (cpus,cpus,jobfile)
	if verbose:
		print cmd
		print program
	try:
		os.system(cmd)
	except: raise RuntimeError("running command \"%s\"" % (cmd))
	os.remove(jobfile)

def Verbose(v, thresh=1): 
	"""	Rreturn a string " verbose" or ""
	"""	
	if v>=thresh: return " verbose"
	else: return ""

class Micrograph:
	"""Define the class for handling each micrograph: storing the necessary information and processing the images in the micrograph"""
	Help={}		# the help messages for each of the option
	def __init__(self):
		self.micrograph			=	"";		Micrograph.Help["micrograph"]		=	"micrograph file name with full path"
		self.micrograph_aligned	=	"";		Micrograph.Help["micrograph_aligned"]	=	"far-from-focus micrograph aligned to the close-to-focus micrograph"
		self.transform_to_first	=	[0,0,0,1];	Micrograph.Help["transform_to_first"]	=	"transform needed for the far-from-focus micrograph to align with the close-to-focus micrograph. in [dx,dy,rot(degree)] format and translation first"
		self.micrograph_merged	=	"";		Micrograph.Help["micrograph_merged"]	=	"merged focal pair micrograph"
		self.micrograph_positive=	None;	Micrograph.Help["micrograph_positive"]	=	"if the micrograph has positive contrast: protein is white, background is black. default to assume \"Project.PositiveContrast\" value"	
		self.boxfile		=	"";		Micrograph.Help["boxfile"]		=	"particle position file name with full path"
		self.imagefile		=	"";		Micrograph.Help["imagefile"]	=	"particle image file name with full path"
		self.imagesize		=	0;		Micrograph.Help["imagesize"]	=	"particle image size in pixel"
		self.imagenum		=	0;		Micrograph.Help["imagenum"]		=	"number of particle images for current micrograph"
		self.image_positive	=	None;	Micrograph.Help["image_positive"]	=	"if the particle images have positive contrast: protein is white, background is black. default to assume \"Project.PositiveContrast\" value"	
		self.apix			=	0.0;	Micrograph.Help["apix"]			=	"particle image sampling step size in Angstrom/pixel"
		self.defocus		=	0.0;	Micrograph.Help["defocus"]		=	"defocus value for current micrograph, in micrometer, positive for underdefocus"
		self.firstpeak		=	0.0;	Micrograph.Help["firstpeak"]	=	"first CTF peak position, in Angstrom"
		self.firstzero		=	0.0;	Micrograph.Help["firstzero"]	=	"first CTF zero position, in Angstrom"
		self.B				=	0.0;	Micrograph.Help["B"]			=	"experimental B factor for current micrograph, in Angstrom^2"
		self.pair			=	None
		# the internal work variables, no need to be visible to outside
		self.__id__			=	""
		self.__nimgfile__	=	""
		self.__nimgsize__	=	0
		self.__goodimgnum__	=	0
		self.__ccmlphase__	=	90.0	# the cross common line phase residual threshold 
		self.__emanworkdir__=	""
		self.__mrcworkdir__	=	""
		self.__ctffile__	=	"ctfparm.txt"
		self.__ctfstr__		=	""
		self.__RefinedDataFile__	=	""
	def __str__(self):
		return self.__repr__()
	def __repr__(self):
		ret="[micrograph]\t# information for a micrograph\n"
		attrs=self.__dict__
		keys=attrs.keys();	keys.sort()
		for attr in keys:
			value=attrs[attr]
			help=""
			if Micrograph.Help.has_key(attr): help="\t# "+Micrograph.Help[attr]
			if value is not None and attr[0:2]!='__' and attr[-2:] != '__' and attr not in ["pair","micrograph_aligned","micrograph_merged"]:
				ret=ret+"%-16s\t=\t%s%s\n" % (attr,value,help)
		if hasattr(self,"pair") and type(self.pair) is type(self) and self.pair:
			ret=ret+"\n[micrograph pair]\t# information for the far-from-focus micrograph of the focal pair\n"
			attrs=self.pair.__dict__
			keys=attrs.keys();	keys.sort()
			for attr in keys:
				value=attrs[attr]
				if value is not None and attr[0:2]!='__' and attr[-2:] != '__' and attr not in ["pair"]:
					ret=ret+"%-16s\t=\t%s\n" % (attr,value)
		return ret
	def __cmp__(self,other):
		"""sort according to defocus, ordered from far-from-focus to close-to-focus"""
		return cmp(other.defocus,self.defocus)
	def Split(self,num=2,verbose=0): 
		"""Split the micrograph into "num" of even parts including the images, return a list of the daughter micrographs"""
		if num<=1: return [self]
		for i in range(num):
			imgfile=os.path.splitext(self.imagefile)[0]+".%02d" % (i)+".img"
			hedfile=os.path.splitext(self.imagefile)[0]+".%02d" % (i)+".hed"
			if os.path.exists(imgfile): os.remove(imgfile)
			if os.path.exists(hedfile): os.remove(hedfile)
		os.chdir(os.path.dirname(self.imagefile))
		cmd="proc2d "+self.imagefile+" "+os.path.splitext(self.imagefile)[0]+".img split="+`num`
		if verbose: print cmd
		os.system(cmd)
		newmgraph=[None]*num
		for i in range(num):
			newmgraph[i]=copy.copy(self)	# shallow copy
			newmgraph[i].imagefile=os.path.splitext(self.imagefile)[0]+".%02d" % (i)+".img"
		p=self.GetFocalPair()
		if p:	# split  the pair micrograph
			pairs=p.Split(num,verbose)
			for i in range(len(pairs)):
				newmgraph[i].pair=pairs[i]
		return newmgraph	
	def ImageInfo(self,usesrcimg=0):
		"""return the image information in a dictionary in order of "num", "size", assuming the images are square"""
		if not self.imagesize or not self.imagenum:
			info=ImageInfo(self.imagefile)
			if info and info["num"]>0 and info["nx"]==info["ny"] and info["nz"]==1: # make sure it is a 2D square image
				self.imagesize=info["nx"]
				self.imagenum=info["num"]
			else:
				return None
		elif not self.__nimgsize__:
			try:
				info=ImageInfo(self.__nimgfile__)
				if info and info["nx"]==info["ny"] and info["nz"]==1:
					self.__nimgsize__=info['nx']
			except:
				pass
		return {"num":self.imagenum, "size":self.imagesize, "csize":self.__nimgsize__} 
	def GetFocalPair(self):
		"""	Check if this has a focal pair micrograph defined. This will require that both micrographs have the same
			number of images of the same size, and the pair must have farther defocus
		"""
		if not hasattr(self,"pair") or not self.pair: return None
		elif not (isinstance(self.pair,type(self))): 
			self.pair = None
			return None
		else:
			info=self.ImageInfo()
			info2=self.pair.ImageInfo()
			if info["num"] != info2["num"] or info["size"] != info2["size"]:
				print "Warning: micrograph \""+self.imagefile+"\" and micrograph \""+self.pair.imagefile+ \
				"\" are defined as focal pairs, but there don't have the same number of particles or " + \
				"particles sizes are different "+`info`+"!="+`info2`+", so micrograph \""+self.pair.imagefile+"\" will be ignored"
				self.pair = None
				return None
			elif self.defocus and self.pair.defocus and self.defocus>=self.pair.defocus:
				print "Warning: micrograph \""+self.imagefile+"\" and micrograph \""+self.pair.imagefile+ \
				"\" are defined as focal pairs, but the pair \""+self.pair.imagefile+"\" does not have larger defocus " + \
				"than the supposed closer-to-focus micrograph \""+self.imagefile+"\""
				self.pair = None
				return None
			else:
				return self.pair
	def AutoFitCTF(self,Project, replace=0, fixedac=0):
		"""Call automatic fitting program fitctf to determine the CTF parameters"""
		if (self.B and self.defocus and not replace) or (replace<2 and os.path.exists(os.path.join(self.__emanworkdir__,self.__ctffile__))): return None 
		if not Project.SFFile or not os.path.exists(Project.SFFile):
			raise IOError("the structure factor file is not specified, so it is not possible to automatically determine the CTF parameters for micrograph \"%s\"" % (self.imagefile))
		os.chdir(self.__emanworkdir__)
		if os.path.exists(self.__ctffile__): os.remove(self.__ctffile__)
		cmd="cd %s ; fitctf %s sf=%s apix=%f V=%f Cs=%f" % (self.__emanworkdir__, self.__nimgfile__, Project.SFFile, Project.__CPixelSize__, Project.Voltage, Project.Cs)
		if fixedac: cmd=cmd+" acfix=%f" % (Project.AmpContrast)
		if Project.Verbose: print cmd
		if not (Project.CPU>1 and Project.RunPar):
			try: 
				os.system(cmd)
			except: raise RuntimeError("during running command \"%s\"" % (cmd))
			return None
		else: 
			return cmd
	def CTFParameters(self, Project, replace=0, fixedac=0):
		"""Return the defocus and B factor for this micrograph. If the CTF parameters are not specified, the automatic fitting program fitctf will be run to find the parameters"""
		if self.B and self.defocus and not replace: return {"B":self.B,"defocus":self.defocus} 
		os.chdir(self.__emanworkdir__)
		if replace>=2 or ((not self.defocus or not self.B) and not os.path.exists(self.__ctffile__)):
			self.AutoFitCTF(Project, fixedac=fixedac)
		if not os.path.exists(self.__ctffile__): raise IOError("cannot find \"ctfparm.txt\" in directory \"%s\"" % (self.__emanworkdir__))
		cf=open(self.__ctffile__)
		ctfpara_str=string.split(cf.readline(),maxsplit=2)[1]
		ctfpara=string.split(ctfpara_str,',',maxsplit=12)
		cf.close()
		self.defocus=math.fabs(float(ctfpara[0]))
		self.B=float(ctfpara[1])
		self.__ctfstr__="%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (ctfpara[0],ctfpara[1],ctfpara[2],ctfpara[3], \
			ctfpara[4],ctfpara[5],ctfpara[6],ctfpara[7],ctfpara[8],ctfpara[9],ctfpara[10])
		return {"B":self.B,"defocus":self.defocus}
	def CtfFirstPeakZero(self, Project):
		"""Find the first CTF peak, zero poistion in Angstrom. This will set the twio attributes "firstpeak", "firstzero" and return them in a list in order of [firstpeak, firstzero]"""
		#print self
		#print "self.defocus:",self.defocus
		ctfcurve=CTFCurve(defocus=-1*self.defocus,B=self.B,ampcontrast=Project.AmpContrast,voltage=Project.Voltage,Cs=Project.Cs,apix=Project.__CPixelSize__,imagesize=Project.__CImageSize__,res=Project.__CPixelSize__*2.0,mode=0)	
		peak=0
		zero=0
		for i in range(1,len(ctfcurve)-1):
			if abs(ctfcurve[i][1])>abs(ctfcurve[i-1][1]) and abs(ctfcurve[i][1])>abs(ctfcurve[i+1][1]): peak=1./ctfcurve[i][0]
			elif ctfcurve[i][1]*ctfcurve[i+1][1]<0:
				zero=1./((ctfcurve[i][0]+ctfcurve[i+1][0])/2.)
				break
		if len(ctfcurve) and (peak or zero):
			self.firstpeak=peak
			if zero: self.firstzero=zero
			else: zero=1./ctfcurve[-1][0] 
			if Project.Verbose:
				print "%s: defocus %f um, CTF 1st peak at %f Angstrom, 1st zero at %f Angstrom" % (self.imagefile,self.defocus,peak,zero)
			return [self.firstpeak,self.firstzero]
		else: 
			return None
	def CtfFirstPeakZeroSlow(self, Project):
		"""Find the first CTF peak, zero poistion in Angstrom. This is obsolete"""
		if self.firstpeak and self.firstzero: return self.firstpeak, self.firstzero
		try:
			if not self.apix: self.apix=Project.PixelSize
			cmd="ctfpz -%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f" % (self.defocus,self.B,1,Project.AmpContrast, \
				0, 0, 0, 0, Project.Voltage, Project.Cs, self.apix)
			fp=os.popen(cmd,'r')
			if Project.Verbose: print cmd
			lines=fp.readlines()
			fp.close()
			peak=string.atof(lines[0])
			zero=string.atof(lines[1])
			if not peak or not zero or zero>=peak: return None
			else:
				self.firstpeak=peak
				self.firstzero=zero	
				return peak, zero
		except:
			return None
	def AutoBoxing(self, Project):
		"""Automatic particle selection using ethan"""
		radius=int(Project.ParticleSize/self.apix+0.5)
		cmdonly=Project.CPU>1 and Project.RunPar
		ret=AutoBoxing(self.micrograph,self.imagefile,self.boxfile,radius,Project.ImageSize,protein_is_white=self.micrograph_positive,verbose=Project.Verbose,cmdonly=cmdonly)
		return ret
	def PrepareDirectories(self, Project):
		"""setup correct directories"""
		if not self.__id__ or not self.__emanworkdir__ or not self.__mrcworkdir__:
			if self.micrograph and os.path.exists(self.micrograph): self.__id__=os.path.basename(os.path.splitext(self.micrograph)[0])
			elif self.imagefile and os.path.exists(self.imagefile): self.__id__=os.path.basename(os.path.splitext(self.imagefile)[0])
			else: raise IOError("Neither \"micrograph\" nor \"imagefile\" is specified or valid for this micrograph %s" % (self) )
			cmgraphdir=os.path.join(Project.ProjectRoot,self.__id__)
			self.__mrcworkdir__=os.path.join(cmgraphdir,"mrc-workdir")
			self.__emanworkdir__=os.path.join(cmgraphdir,"eman-workdir")
		if not os.path.exists(self.__mrcworkdir__): os.makedirs(self.__mrcworkdir__)
		if not os.path.exists(self.__emanworkdir__): os.makedirs(self.__emanworkdir__)

		# set up the focal pair if it exists
		if self.pair and type(self.pair)==type(self): self.pair.PrepareDirectories(Project)
	def PrepareWorkingImages(self, Project, cycle=0):
		"""Make sure the working copy of images exist and contains correct information"""
		if not self.imagefile or not os.path.exists(self.imagefile): raise ValueError("Micrograph.PrepareWorkingImages() requires \"imagefile\" set for correct function")
		imgbase=os.path.basename(os.path.splitext(self.imagefile)[0])
		if not self.__nimgfile__:
			self.__nimgfile__=os.path.join(self.__emanworkdir__,imgbase+".img")
		
		self.__RefinedDataFile__=os.path.basename(os.path.splitext(self.imagefile)[0])+"-refined-to-res-%.2fAngstrom.dat" % (Project.MRCRefine[cycle])
		self.__RefinedDataFile__=os.path.join(self.__mrcworkdir__,self.__RefinedDataFile__)
		
		# resolution and defocus dependent phase residue threshold
		if Project.CCMLPWeight and self.defocus:
			self.__ccmlphase__=Project.CCMLPLimits[0]+math.fabs((4.-self.defocus)/4.)*float(cycle)/(len(Project.MRCRefine)-1.0)*(Project.CCMLPLimits[1]-Project.CCMLPLimits[0])
		else:
			self.__ccmlphase__=Project.CCMLPLimits[0]+float(cycle)/float(len(Project.MRCRefine))*(Project.CCMLPLimits[-1]-Project.CCMLPLimits[0])

		if not self.apix: self.apix=Project.PixelSize

		#if not os.path.exists(self.imagefile):
		#	if not Project.MultSampling and not os.path.islink(self.__nimgfile__) and os.path.exists(self.__nimgfile__): # could ignore source images only when the working copy of images exist and other processing options allow
		#		print "Warning: cannot find file \""+self.imagefile+"\""
		#	else:
		#		raise IOError("cannot find file \""+self.imagefile+"\"")
		#
		if not self.imagesize or not self.__nimgsize__ or not self.imagenum:
			if not self.ImageInfo():
				raise ValueError("please make sure image files \""+self.imagefile+"\" and \""+self.__nimgfile__+"\" are fine")

		if self.image_positive==None: self.image_positive=Project.PositiveContrast
		
		nimgfile=self.__nimgfile__
		nhedfile=os.path.join(self.__emanworkdir__,imgbase+".hed")
		oimgfile=os.path.join(os.path.dirname(self.imagefile),imgbase+".img")
		ohedfile=os.path.join(os.path.dirname(self.imagefile),imgbase+".hed")
		
		cmd=""
		if not self.Processed(Project,cycle=cycle):
			from stat import ST_UID
			myid=os.getuid()
			hedid=os.stat(ohedfile)[ST_UID]
			if os.path.dirname(self.imagefile) != self.__emanworkdir__ and not Project.MultSampling and self.apix == Project.PixelSize \
			   and self.image_positive and Project.ImageSize == self.ImageInfo()["size"]:
				if myid == hedid:
					if not os.path.exists(nhedfile): os.symlink(ohedfile,nhedfile)
					if not os.path.exists(nimgfile): os.symlink(oimgfile,nimgfile)
				else:
					from shutil import copyfile
					if not os.path.exists(nimgfile): os.symlink(oimgfile,nimgfile)
					#if not os.path.exists(nimgfile): copyfile(oimgfile,nimgfile) 	# temp solution
					#if not os.path.exists(nhedfile): copyfile(ohedfile,nhedfile)
					if not os.path.exists(nhedfile): cmd=cmd+"cp %s %s" % (ohedfile,nhedfile)
			else:
				shrink=Project.ShrinkFactor(cycle=cycle)
				if cycle: lshrink=Project.ShrinkFactor(cycle=cycle-1)
				else: lshrink=shrink
				
				if lshrink!=shrink or not os.path.exists(nhedfile) or not os.path.exists(nimgfile):
					cmd=cmd+"cd %s ;" % self.__emanworkdir__
					if lshrink != shrink:
						temportcen="%s-ortcen-at-cycle-of-%.2f-Angstrom.dat" % (imgbase,Project.MRCRefine[cycle])
						cmd=cmd+"Imagic2OrtCen %s %s ; rm %s %s ;" % (nimgfile,temportcen,nhedfile,nimgfile)
					elif os.path.exists(nhedfile): os.remove(nhedfile)
					elif os.path.exists(nimgfile): os.remove(nimgfile)
					cmd=cmd+"proc2d %s %s edgenorm clip=%d,%d" % (oimgfile, os.path.basename(nimgfile), Project.__CImageSize__,Project.__CImageSize__)
					if not self.image_positive: cmd=cmd+" invert"
					if shrink!=1 or self.apix != Project.__CPixelSize__: cmd=cmd+" scale=%f" % (self.apix/Project.__CPixelSize__)
					if lshrink!=shrink: cmd=cmd+"; OrtCen2Imagic %s %s censcale=%f" % (temportcen,nimgfile,float(lshrink)/float(shrink))

		# set up the focal pair if it exists
		p=self.GetFocalPair()
		if p: 
			tcmd=p.PrepareWorkingImages(Project,cycle)
			if tcmd: cmd=cmd+"\n"+tcmd

		if not (Project.CPU>1 and Project.RunPar) and cmd:
			if Project.Verbose: print cmd
			os.system(cmd)
			return None
		elif Project.CPU>1 and Project.RunPar and cmd:
			return cmd
		else:
			return None		
	def Processed(self, Project, cycle=0):
		"""Check if this micrograph is already processed for this "cycle" """
		if os.path.exists(self.__RefinedDataFile__): return 1
		else:	return 0
	def WaveProjectionMatching(self, Project, cycle=0):
		"""Determine the initial orientation using Wavelet-assisted projection matching method"""
		p=self.GetFocalPair() 
		if p: m=p
		else: m=self
		os.chdir(m.__emanworkdir__)
		stamp="Cycle-of-%.2fAngstrom-is-done" % (Project.MRCRefine[cycle])
		#if os.path.exists(stamp) and os.path.getmtime(stamp)>os.path.getmtime(hedfile): 
		if os.path.exists(stamp): 
			if Project.Verbose:
				print "Micrograph \"%s\" is already projection-matching processed for cycle of %.2f Angstrom resolution, skipped" % (os.path.splitext(os.path.basename(m.imagefile))[0],Project.MRCRefine[cycle])
			return None
		
		#angrange=(Project.MRCRefine[cycle]*2./Project.ParticleSize*180/math.pi)*1.5
		#mask=Project.ParticleSize*1.05/Project.__CPixelSize__/2.0+(1.0-float(cycle)/float(len(Project.MRCRefine)-1))*(m.ImageInfo()["csize"]-Project.ParticleSize*1.05/Project.__CPixelSize__)/2.0
		mask=Project.__CMaskRadius__
		cmd="cd %s ; " % os.path.abspath(m.__emanworkdir__)
		if cycle==0 and Project.CenAlignOpt in [2,3]:	# center alignment options
			#maxshift=(m.ImageInfo()["size"]-Project.ParticleSize/m.apix)*m.apix/Project.__CPixelSize__/2.0
			maxshift=(Project.RefineMaskFactor[0]-1.0)*(Project.ParticleSize/2.)/Project.__CPixelSize__
			if Project.CenAlignOpt==2:
				cenref=os.path.join(Project.__WAVEProjDir__,"proj-rotavg.img")
				#cmd=cmd+"cenalign "+os.path.join(m.__emanworkdir__,os.path.basename(m.imagefile))
				cmd=cmd+"cenalign "+m.__nimgfile__
				if os.path.exists(cenref):
					cmd=cmd+" "+cenref
				else:
					cmd=cmd+" "+Project.__MRCProjFile__+" saveref="+cenref
			elif Project.CenAlignOpt==3:
				#cmd=cmd+"cenalign2 "+os.path.join(m.__emanworkdir__,os.path.basename(m.imagefile))+ " " + \
				cmd=cmd+"cenalign2 "+m.__nimgfile__+ " " + Project.__MRCProjFile__
			else:
				raise ValueError("\"Project.CenAlignOpt\" only supports values 0, 1, 2, or 3! %s is not supported" % (Project.CenAlignOpt))
			if Project.ApplyCTF:
				if m.defocus>1.0:		cmd=cmd+" ctf="+os.path.join(m.__emanworkdir__,m.__ctffile__) 
				else:				cmd=cmd+" unctf="+os.path.join(m.__emanworkdir__,m.__ctffile__)
			if not Project.SMCTFCorrect: cmd=cmd+" invert"
			cmd=cmd+" maxshift="+`maxshift`+" apix="+`Project.__CPixelSize__`+" filter="+`Project.ParticleSize`+","+`Project.MRCRefine[-1]`+",0"+Verbose(Project.Verbose,thresh=2)+";"
		cmd=cmd+"WaveOrt "+m.__nimgfile__+" "+ Project.__WAVEProjFile__+" ignorerefid tree=2"+" mask="+`mask`
			#os.path.join(Project.__WAVEProjDir__,"proj.img")+" cpu="+`Project.CPU`+" ignorerefid tree="+`2.5/Project.__WAVEProjStep__`+" mask="+`mask`
		if Project.ApplyCTF: cmd=cmd+" ctf="+os.path.join(m.__emanworkdir__,m.__ctffile__)
		if cycle==0 and Project.CenAlignOpt==1: cmd=cmd+" ignorecen"
		if cycle>0:	cmd=cmd+" minN=1"
		#cmd=cmd+Verbose(Project.Verbose,thresh=2)+"; touch "+stamp
		cmd=cmd+Verbose(Project.Verbose,thresh=2)+" touch="+stamp
		if p: cmd=cmd+"; copyHeader %s %s" % (p.__nimgfile__, self.__nimgfile__)
		if Project.RunPar and Project.CPU>=2:	return cmd
		else:
			if Project.Verbose: print cmd
			try:
				os.system(cmd)
			except: raise RuntimeError("during running command \"%s\"" % (cmd) )
			return None
	def SCMLEstimateOrientation(self,Project):
		"""Initial estimatation of the center and orientation for each particle using the self-commonline algorithm""" 
		p=self.GetFocalPair() 	
		if p : m=p
		else: m=self	
		
		os.chdir(m.__mrcworkdir__)
		res=Project.MRCRefine[0]
		out=os.path.basename(os.path.splitext(m.imagefile)[0])+"-initial-ortcen-to-res-%.2fAngstrom.dat" % (res)
		out2=os.path.basename(os.path.splitext(m.imagefile)[0])+"-initial-ortcen-to-res-%.2fAngstrom-global-refined.dat" % (res)

		if os.path.exists(out2) and os.path.getsize(out2): 
			if Project.Verbose:
				print "Micrograph \"%s\" is already self-commonline estimated for the initial orientation and center parameters, skipped" % (os.path.splitext(os.path.basename(m.imagefile))[0])
			return None
			
		if m.firstzero>res: res=m.firstzero	
		#mask=m.ImageInfo()["csize"]/2.0
		mask=Project.__MaskRadius__
		low=int(m.ImageInfo()["csize"]*Project.__CPixelSize__/(Project.ParticleSize*1.25)+0.5)
		if low<3:	low=3
		high=int(m.ImageInfo()["csize"]*Project.__CPixelSize__/res+0.5)
		rot=float(res)/(Project.ParticleSize/2.)*180./math.pi/2.
		phase=Project.CCMLPLimits[0]
		rawimg=m.__nimgfile__
			# to deal with focal pair, we need transfer the parameters back to the in-focus micrograph
		rawimg2=self.__nimgfile__
		
		cmd="cd %s ; " % os.path.abspath(m.__mrcworkdir__)
		cmd=cmd+"proc2d "+rawimg+" mrc mrc"
		if not Project.SMCTFCorrect: cmd=cmd+" invert" 
		cmd=cmd+" ; ortAll.py 100.mrc "+`100+m.ImageInfo()["num"]`+".mrc"+" --low="+`low`+" --high="+`high`+" --mask1="+`mask`+" --mask2="+`mask`+" --apix="+`Project.__CPixelSize__`
		cmd=cmd+"; eliminateort.py "+os.path.join(Project.__MRCProjDir__,Project.__MRCDataFile__)+" 100.log "+`100+m.ImageInfo()["num"]`+".log"+\
			" --phase="+`phase`+" --low="+`low`+" --high="+`high`+" --imgsize="+`m.ImageInfo()["csize"]`+" --out="+out
		cmd=cmd+" ; globalRefine.py "+out+" "+out2+" --low="+`low`+" --high="+`high`+" --step="+`rot`
		cmd=cmd+"; OrtCen2Imagic "+out2+" "+rawimg+" phase="+`phase`+Verbose(Project.Verbose,thresh=2)
		cmd=cmd+"; rm *log *mrc *mas *cen *.tnf"
		if p: 
			cmd=cmd+"; OrtCen2Imagic "+out2+" "+rawimg2+" phase="+`phase`+Verbose(Project.Verbose,thresh=2)
			#if cycle==Project.WaveCycles-1:
			#	maxshift=(self.ImageInfo()["size"]-Project.ParticleSize/self.apix)/2.0
			#	eman3dmap=os.path.join(Project.__MRC3DDir__,"icos5f-full.mrc")
			#	projimg=os.path.join(self.__emanworkdir__,"proj.img")
			#	projhed=os.path.join(self.__emanworkdir__,"proj.hed")
			#	if os.path.exists(projimg): os.remove(projimg)
			#	if os.path.exists(projhed): os.remove(projhed)
			#	info=ImageInfo(rawimg2)
			#	if Project.CPU>info["num"]: cpu2=info["num"]
			#	else:			cpu2=Project.CPU
			#	cmd=cmd+"; projecticosP "+eman3dmap+" out="+projimg+" inort2="+rawimg2+" cpu="+`cpu2`+" clip="+`info["nx"]`+Verbose(Project.Verbose,thresh=2)
			#	if (cycle==0 and not Project.SMCTFCorrect) or (cycle>0 and Project.MRCRefine[cycle-1]>Project.SMResolution and not Project.SMCTFCorect) or (cycle>0 and Project.MRCRefine[cycle-1]<Project.SMResolution and not Project.CTFCorrect): cmd=cmd+" invert" 
			#	cmd=cmd+"; cenalign2 "+rawimg2+" "+projimg+" exact maxshift="+`maxshift`+Verbose(Project.Verbose,thresh=2)	
			#	if Project.ApplyCTF:	
			#		if p.defocus>1.0:		cmd=cmd+" ctf="+os.path.join(self.__emanworkdir__,self.__ctffile__)
			#		else:				cmd=cmd+" unctf="+os.path.join(self.__emanworkdir__,self.__ctffile__)
			#	cmd=cmd+"; rm %s %s" % (projimg,projhed)
		if Project.Verbose: print cmd
		if Project.RunPar:	return cmd
		else:	
			os.system(cmd)
			return None
	def CCMLRefine(self,Project,cycle=0):
		"""Refine the center and orientation for each particle using the cross-commonline algorithm""" 
		p=self.GetFocalPair()
		if p and cycle<Project.WaveCycles: m=p
		else: m=self
		os.chdir(m.__mrcworkdir__)
		m.__RefinedDataFile__=os.path.basename(os.path.splitext(m.imagefile)[0])+"-refined-to-res-%.2fAngstrom.dat" % (Project.MRCRefine[cycle])
		m.__RefinedDataFile__=os.path.join(m.__mrcworkdir__,m.__RefinedDataFile__)
		if os.path.exists(m.__RefinedDataFile__) and os.path.getsize(m.__RefinedDataFile__): 
			if Project.Verbose:
				print "Micrograph \"%s\" is already cross-commonline refined for cycle of (%d of %d) %.2f Angstrom resolution, skipped" % (os.path.splitext(os.path.basename(m.imagefile))[0],cycle,len(Project.MRCRefine),Project.MRCRefine[cycle])
			return None
			
		#mask=Project.ParticleSize*1.05/Project.__CPixelSize__/2.0+(1.0-float(cycle)/float(len(Project.MRCRefine)-1))*(m.ImageInfo()["csize"]-Project.ParticleSize*1.05/Project.__CPixelSize__)/2.0
		mask=Project.__CMaskRadius__
		low=int(m.ImageInfo()["csize"]*Project.__CPixelSize__/(Project.ParticleSize*1.25)+0.5)
		if low<3:	low=3
		rot=float(Project.MRCRefine[cycle])/(Project.ParticleSize/2.)*180./math.pi/2.
		#rot=float(Project.MRCRefine[cycle])/(Project.ParticleSize/2.)*180./math.pi*0.75
		trans=float(Project.MRCRefine[cycle])/8.0/Project.__CPixelSize__

		phase=90.0	# set the theshold high here and constrains later on during reconstruction
		#rawimg=os.path.join(m.__emanworkdir__,os.path.basename(m.imagefile))
		rawimg=m.__nimgfile__
			# to deal with focal pair, we need transfer the parameters back to the in-focus micrograph
		#rawimg2=os.path.join(self.__emanworkdir__,os.path.basename(self.imagefile))
		rawimg2=self.__nimgfile__
		
		out=os.path.join(m.__mrcworkdir__,os.path.basename(os.path.splitext(m.imagefile)[0])+"-refined-to-res-%.2fAngstrom.dat" % (Project.MRCRefine[cycle]))
		m.RefineDataFile=out
		ortcen="Wave-ort-cen-cycle-of-%.2fAngstrom.dat" % (Project.MRCRefine[cycle])
		high=int(m.ImageInfo()["csize"]*Project.__CPixelSize__/Project.MRCRefine[cycle]+0.5)
		refine_cycles=int(Project.CCMLCycles[0]+float(cycle)/float(len(Project.MRCRefine)-1)*(Project.CCMLCycles[-1]-Project.CCMLCycles[0]))
		
		cpu=Project.CPU
		if Project.RunPar: cpu=1

		cmd="cd %s ; " % os.path.abspath(m.__mrcworkdir__)
		cmd=cmd+"fftimagic "+rawimg+" outort="+ortcen+" invert mask="+`mask`+" path="+m.__mrcworkdir__+Verbose(Project.Verbose,thresh=2) 
		cmd=cmd+" ; refineAll.py "+ortcen+" "+Project.__MRCDataFile__+ \
			" --apix="+`Project.__CPixelSize__`+" --low="+`low`+" --high="+`high`+" --cpu="+`cpu`+" --out="+out+" --trans="+`trans`+" --rot="+`rot`+" --phase="+`phase`+" --cycle="+`refine_cycles`
		#if (Project.SMCTFCorrect and cycle==0) or (Project.CTFCorrect and cycle!=0): cmd=cmd+" --ctf="+Project.__MRCCTFFile__
		if (cycle<Project.WaveCycles and Project.SMCTFCorrect) or (cycle>=Project.WaveCycles and Project.MRCRefine[cycle-1]>Project.SMResolution and Project.SMCTFCorrect) or (cycle>=Project.WaveCycles and Project.MRCRefine[cycle-1]<Project.SMResolution and Project.CTFCorrect): cmd=cmd+" --ctf="+Project.__MRCCTFFile__
		cmd=cmd+"; OrtCen2Imagic "+out+" "+rawimg+" phase="+`phase`+Verbose(Project.Verbose,thresh=2)
		cmd=cmd+"; rm *log emicograd.dat %s" % (os.path.splitext(os.path.basename(Project.__MRCDataFile__))[0]+"*.tnf")
		if p and cycle<Project.WaveCycles: 
			cmd=cmd+"; OrtCen2Imagic "+out+" "+rawimg2+" phase="+`phase`+Verbose(Project.Verbose,thresh=2)
			#if cycle==Project.WaveCycles-1:
			#	maxshift=(self.ImageInfo()["size"]-Project.ParticleSize/self.apix)/2.0
			#	eman3dmap=os.path.join(Project.__MRC3DDir__,"icos5f-full.mrc")
			#	projimg=os.path.join(self.__emanworkdir__,"proj.img")
			#	projhed=os.path.join(self.__emanworkdir__,"proj.hed")
			#	if os.path.exists(projimg): os.remove(projimg)
			#	if os.path.exists(projhed): os.remove(projhed)
			#	info=ImageInfo(rawimg2)
			#	if Project.CPU>info["num"]: cpu2=info["num"]
			#	else:			cpu2=Project.CPU
			#	cmd=cmd+"; projecticosP "+eman3dmap+" out="+projimg+" inort2="+rawimg2+" cpu="+`cpu2`+" clip="+`info["nx"]`+Verbose(Project.Verbose,thresh=2)
			#	if (cycle==0 and not Project.SMCTFCorrect) or (cycle>0 and Project.MRCRefine[cycle-1]>Project.SMResolution and not Project.SMCTFCorect) or (cycle>0 and Project.MRCRefine[cycle-1]<Project.SMResolution and not Project.CTFCorrect): cmd=cmd+" invert" 
			#	cmd=cmd+"; cenalign2 "+rawimg2+" "+projimg+" exact maxshift="+`maxshift`+Verbose(Project.Verbose,thresh=2)	
			#	if Project.ApplyCTF:	
			#		if p.defocus>1.0:		cmd=cmd+" ctf="+os.path.join(self.__emanworkdir__,self.__ctffile__)
			#		else:				cmd=cmd+" unctf="+os.path.join(self.__emanworkdir__,self.__ctffile__)
			#	cmd=cmd+"; rm %s %s" % (projimg,projhed)
		if Project.CPU>=2 and Project.RunPar:	return cmd
		else:
			if Project.Verbose: print cmd
			try:
				os.system(cmd)		
			except: raise RuntimeError("during running command \"%s\"" % (cmd) )
			return None

class Project:
	"""Define the overall project information, parameters controlling the processing behavior, and the various statistical assessments of the processing results"""
	Help={}		# the help messages for each of the option
	def __init__(self):
		self.Sample			=	'Default Sample';	Project.Help["Sample"]			=	"short description of the sample"
		self.MolMass		=	0.0;				Project.Help["MolMass"]			=	"total molecular mass of the sample, in kD"
		self.ProjectFile	=	"";					Project.Help["ProjectFile"]		=	"file name of current project file"
		self.ProjectRoot	=	os.path.abspath(os.path.join(os.curdir,"ProcessingRoot"));	Project.Help["ProjectRoot"]	=	"directory that all processing would be run in. It should be a dedicated directory and does not include source data, otherwise the source data could be erased"
		self.StartModel		=	"";					Project.Help["StartModel"]		=	"starting model for the refinement. It would be automatically generated if not available"
		self.SFFile			=	"";					Project.Help["SFFile"]			=	"structural factor text file in two column format: inverse freququency and scattering intensity. Required for automatic CTF fitting"
		self.SMCTFCorrect	=	0;					Project.Help["SMCTFCorrect"]	=	"if the start model is CTF-corrected"
		self.SMResolution	=	0.0;				Project.Help["SMResolution"]	=	"resolution of the start model in Angstrom"
		self.SMPixelSize	=	0.0;				Project.Help["SMPixelSize"]		=	"sampling size for the start model, in Angstrom/pixel"
		self.ResRange		=	[30,8];				Project.Help["ResRange"]		=	"resolution range for the refinement, in format of \"[low, high]\" where low and high are in Angstrom" 
		self.RefineMore		=	0.0;				Project.Help["RefineMore"]		=	"refine further from \"Project.ResRange[high]\" to the given resolution, in Angstrom"
		self.MRCRefine		=	[];					Project.Help["MRCRefine"]		=	"list of resolutions will be refined in order of low to high resolutions in Angstrom. This overrides \"Project.ResRange\" and \"Project.RefineMore\" options"
		self.CCMLPWeight	=	1;					Project.Help["CCMLPWeight"]		=	"if the phase residual threshold is weighted by the defocus of each micrograph: more tolerant for closer to focus"
		self.CCMLPLimits	=	[50,80];			Project.Help["CCMLPLimits"]		=	"phase residual threshold in degree"
		self.CCMLOLimits	=	0.0	;				Project.Help["CCMLOLimits"]		=	"refinement orientation change threshold for a particle to be included in 3D reconstruction, in degree"
		self.CCMLCLimits	=	0.0	;				Project.Help["CCMLCLimits"]		=	"refinement center change threshold for a particle to be included in 3D reconstruction, in pixel"
		self.CCMLCycles		=	[4,4];				Project.Help["CCMLCycles"]		=	"refinement cycles performed during each cross commonline refinement. Will graudually change from first number to second number of cycles. default to [4,4]."	
		self.WaveProjFactor	=	1.0;				Project.Help["WaveProjFactor"]	=	"projection step size factor for Wavelet projection matching: 1-default, >1-more projections, <1-fewer projections"
		self.CCMLProjFactor	=	1.0;				Project.Help["CCMLProjFactor"]	=	"projection step size factor for Cross Common Line refinement: 1-default, >1-more projections, <1-fewer projections"
		self.RefineCycleSize=	[];					Project.Help["RefineCycleSize"]	=	"refinement resolution forward step sizes in format of \"[large,samll]\" that starts with larger steps, gradually decreases the step sizes, and ends with smaller steps. This is automatically computed if not given"
		self.RefineMaskFactor=	[];					Project.Help["RefineMaskFactor"]=	"the factor multiplied to the particles size for determining the mask sizes. default to [1.25,1.05], start with larger masks and gradually decrease to the smaller masks"
		self.DefocusRange	=	[];					Project.Help["DefocusRange"]	=	"specifies the micrographs in the given range of defocuses for processing, in format of \"[close, far]\" where \"close\" and \"far\" are in micrometer"
		self.BfactorRange	=	[];					Project.Help["BfactorRange"]	=	"specifies the micrographs in the given range of B factors for processing, in format of \"[small, large]\" where \"small\" and \"larger\" are in micrometer"
		self.Voltage		=	400.0;				Project.Help["Voltage"]			=	"accelerating voltage in KeV"
		self.Cs				=	4.1;				Project.Help["Cs"]				=	"spherical abberation coefficient in mm"
		self.AmpContrast	=	0.07;				Project.Help["AmpContrast"]		=	"amplitude contrast fraction"
		self.MapSize	 	=	0;					Project.Help["MapSize"]			=	"specifies the 3D reconstruction map size in pixels, should be odd integer"
		self.ImageSize	 	=	360;				Project.Help["ImageSize"]		=	"size of the boxed particle images in pixels, should be even integers friendly for FFT"
		self.IgnoreSrcImgs	=	0;					Project.Help["IgnoreSrcImgs"]	=	"if ignore the checking of source images"
		self.IgnoreFocalPair=	0;					Project.Help["IgnoreFocalPair"]	=	"if ignore the second far-from-focus micrograph and only make use of the first close-to-focus micrograph" 
		self.PositiveContrast	=0;					Project.Help["PositiveContrast"]=	"if the source images should be inverted so that \"protein\" would be white before processing starts"
		self.PixelSize	 	=	2.7;				Project.Help["PixelSize"]		=	"pixel size of the 3D maps and working copy of images for the finest image samplings in Angstrom/pixel"
		self.MultSampling	=	0;					Project.Help["MultSampling"]	=	"if sampling step sizes at different refinement stages will be varied"
		self.ParticleSize	=	700.0;				Project.Help["ParticleSize"]	=	"particle diameter in Angstrom"
		self.InnerRadius	=	276.0;				Project.Help["InnerRadius"]		=	"inner radius of capsid in Angstrom, 0 for filled capsid"
		self.ICFactor		=	1.0;				Project.Help["ICFactor"]		=	"inner face curvature factor relative to the inner radius for masking: 0-planar, >1-slighly curved, 1-spherical, <1-highly curved mask"
		self.IMaskSlope		=	10.0;				Project.Help["IMaskSlope"]		=	"inner mask slope of Gaussian function with this half width in Angstrom"
		self.CenAlignOpt	=	0;					Project.Help["CenAlignOpt"]		=	"center alignment option: 0-no center alignment and reuse current center values, 1-no center alignment and ignore current center values, 2-against rotation average, 3-against a group of random projections"
		self.Symmetrize		=	1;					Project.Help["Symmetrize"]		=	"if the 3D models are symmetrized before projection to remove the icosahedral reconstruction anisotropic background"
		self.WaveInMask		=	0;					Project.Help["WaveInMask"]		=	"if apply inner mask to 3D models before generating projections for Wavelet projection matching"
		self.CCMLInMask		=	0;					Project.Help["CCMLInMask"]		=	"if apply inner mask to 3D models before generating projections for Cross Common Line refinement"
		self.FSCInMask		=	1;					Project.Help["FSCInMask"]		=	"if apply inner mask to 3D models before Fourier Shell Correlating two reconstructions"
		self.CTFCorrect		=	1;					Project.Help["CTFCorrect"]		=	"if apply CTF correction during 3D reconstruction"
		self.ApplyCTF		=	1;					Project.Help["ApplyCTF"]		=	"if apply CTF for Wavelet projection matching and center alignment option 2"
		self.FixedBFactor	=	0.0;				Project.Help["FixedBFactor"]	=	"if >0, use this constant B factor during 3D reconstruction for all micrographs, otherwise use individual B factors from each micrograph"
		self.AddExtraB		=	0.0;				Project.Help["AddExtraB"]		=	"Experimental. will add this extra B factor to the individual B factors for each micrograph"
		self.WaveCycles		=	1;					Project.Help["WaveCycles"]		=	"Experimental. specifies number of cycles of Wavelet projection matching"
		self.CPU			=	1;					Project.Help["CPU"]				=	"maximal number of CPUs to use during processing"
		self.RunPar			=	1;					Project.Help["RunPar"]			=	"use EMAN runpar mechanism for parallel processing instead of the problematic and obsolete SGI loop parallelism"
		self.TTest			=	0;					Project.Help["TTest"]			=	"if compute Fourier Shell Correlation of 3D reconstructions for each refinement cycle"
		self.DeepTTestRes	=	0.0;				Project.Help["DeepTTestRes"]	=	"refinement cycle closest to this resolution will have multpile level of FSC computed. 0 for the last cycle"
		self.DeepTTestLevel	=	1;					Project.Help["DeepTTestLevel"]	=	"number of levels for the multiple level FSC"
		self.Mail			=	"";					Project.Help["Mail"]			=	"email addresses to send processing progress reports after each refinement cycle. multiple addresses could be specified"
		self.Verbose		=	0;					Project.Help["Verbose"]			=	"control the verbosity of processing output: 0-silent, 1-output processing commands, 2-plus output from each of commands"

		self.Help			=	Project.Help
		
		self.Data			=	[]	# the micrographs

		# the internal work variables, no need to be visible to outside
		self.__MRC3DDir__	 	=	''
		self.__MRCProjDir__	 	=	''
		self.__MRCDataFile__ 	=	''
		self.__MRCCTFFile__	 	=	"CTF-Bfactor-curve.dat"
		self.__WAVEProjDir__ 	= 	''
		self.__WAVEProjFile__	=	''
		self.__MRCProjFile__	=	''
		self.__EMANDir__	 	=	''
		self.__CPixelSize__ 	=	self.PixelSize  # the pixel step size for current cycle
		self.__CImageSize__ 	=	self.ImageSize  # the image size for current cycle
		self.__CMapSize__   	=	self.MapSize
		self.__CMaskRadius__		=	self.__CImageSize__/2  # current mask radius
		self.__DeepTTestCycle__	=	-1
	def __str__(self):
		return self.__repr__()
	def __repr__(self):
		ret="[Project]\t# specify overall information for the project\n" 
		attrs=self.__dict__
		keys=attrs.keys();	keys.sort()
		for attr in keys:
			value = attrs[attr]
			help=""
			if Project.Help.has_key(attr): help="\t# "+Project.Help[attr]
			if value!=None and attr[0:2]!='__' and attr[-2:] != '__' and attr not in ["Data","Help"]:
					ret=ret+"%-16s\t=\t%s%s\n" % (attr,value,help)
		pairhelp=0	# keep track if the help info has been written to the first pair micrograph
		for m in range(len(self.Data)):
			if m==0: ret=ret+"\n[micrograph %d]\t# information for a micrograph\n" % (m) 
			else: ret=ret+"\n[micrograph %d]\n" % (m)
			attrs=self.Data[m].__dict__
			keys=attrs.keys();	keys.sort()
			for attr in keys:
				value=attrs[attr]
				help=""
				if m==0 and Micrograph.Help.has_key(attr): help="\t# "+Micrograph.Help[attr]
				if value is not None and attr[0:2]!='__' and attr[-2:] != '__' and not (attr == "image_positive" and value==1) and not (attr == "micrograph_positive" and value==1) and \
				   attr not in ["pair","boxfile","micrograph","micrograph_aligned","micrograph_merged","transform_to_first"]:
					ret=ret+"%-16s\t=\t%s%s\n" % (attr,value,help)
			if hasattr(self.Data[m],"pair") and type(self.Data[m].pair) is type(self.Data[m]) and self.Data[m].pair:
				ret=ret+"\n[micrograph %d pair]" % (m)
				if not pairhelp:
					ret=ret+"\t# information for the far-from-focus micrograph of the focal pair\n"
				else: ret=ret+"\n"
				attrs=self.Data[m].pair.__dict__
				keys=attrs.keys();	keys.sort()
				for attr in keys:
					value=attrs[attr]
					help=""
					if not pairhelp and Micrograph.Help.has_key(attr): help="\t# "+Micrograph.Help[attr]
					if value is not None and attr[0:2] !='__' and attr[-2:] != '__' and not (attr == "image_positive" and value==1) and not (attr == "micrograph_positive" and value==1) and \
					   attr not in ["image_positive","boxfile","micrograph","micrograph_aligned","micrograph_merged"] and not (attr == "transform_to_first" and value == [0,0,0,1] ):
						ret=ret+"%-16s\t=\t%s%s\n" % (attr,value,help)
				if not pairhelp: pairhelp=1
		return ret
	def Update(self, update_dict={}):
		""" Update the attributes according to the input dictionary. mainly for xml-rpc call"""
		for k in update_dict.keys():
			if hasattr(self.__dict__,k) and type(self.__dict__[k])==type(update_dict[k]):
				self.__dict__[k]=update_dict[k]
	def AddMicrograph(self, micrograph=None):
		if not micrograph: m=Micrograph()
		else: m=micrograph
		self.Data.append(m)
	def Processed(self, cycle=0):
		ret=1
		for m in self.Data:
			ret=ret and m.Processed(self, cycle=cycle)
			if not ret: return 0
		return 1
	def ShrinkFactor(self,cycle=0):
		if not self.MultSampling: return 1
		else:
			f=int(self.MRCRefine[cycle]/(4.0*self.PixelSize))
			if not f: f=1
			return f
	def SortMicrograph(self, var="defocus", order="descend"):
		if order=="descend":
			if var=="defocus":
				self.Data.sort(lambda m1, m2: cmp(m2.defocus,m1.defocus))
			elif var=="imagenum":
				self.Data.sort(lambda m1, m2: cmp(m2.ImageInfo()["num"],m1.ImageInfo()["num"]))
			elif var=="B":
				self.Data.sort(lambda m1, m2: cmp(m2.B,m1.B))
		elif order=="ascend":
			if var=="defocus":
				self.Data.sort(lambda m1, m2: cmp(m1.defocus,m2.defocus))
			elif var=="imagenum":
				self.Data.sort(lambda m1, m2: cmp(m1.ImageInfo()["num"],m2.ImageInfo()["num"]))
			elif var=="B":
				self.Data.sort(lambda m1, m2: cmp(m1.B,m2.B))
		else:
			pass	
	def IgnoreSourceImages(self):
		"""Just to get around if the source images are removed. This should not be abused"""
		for m in self.Data:
			mgraph=os.path.basename(os.path.splitext(m.imagefile)[0])
			cmgraphdir=os.path.join(self.ProjectRoot,mgraph)
			emanworkdir=os.path.join(cmgraphdir,"eman-workdir")
			imgbase=os.path.basename(os.path.splitext(m.imagefile)[0])
			m.__nimgfile__=os.path.join(emanworkdir,imgbase+".img")
			m.imagefile=m.__nimgfile__
			if hasattr(m,"pair") and (isinstance(m.pair,type(m))):
				mgraph=os.path.basename(os.path.splitext(m.pair.imagefile)[0])
				cmgraphdir=os.path.join(self.ProjectRoot,mgraph)
				emanworkdir=os.path.join(cmgraphdir,"eman-workdir")
				imgbase=os.path.basename(os.path.splitext(m.pair.imagefile)[0])
				m.pair.__nimgfile__=os.path.join(emanworkdir,imgbase+".img")
				m.pair.imagefile=m.pair.__nimgfile__
	def RunParCommands(self, commands, jobfile=None):
		"""Use RunPar to run given list of commands in parallel"""
		if commands:
			if not jobfile: jobfile=os.path.join(self.ProjectRoot,"savr-jobs.runpar")
			jobfp=open(jobfile,"w")
			for c in string.split(commands,"\n"):
				if c: jobfp.write("runpar.py \"%s\"\n" % c)
			jobfp.close()
			cmd="runpar proc=%d,%d file=%s" % (self.CPU,self.CPU,jobfile)
			if self.Verbose:
				print commands
				print cmd
			try:
				os.system(cmd)
			except: raise RuntimeError("running command \"%s\"" % (cmd))
			os.remove(jobfile)
	def ReadProjectFile(self,file):
		"""Read the project information from a file"""
		if not os.path.exists(file): raise IOError("project file \"%s\" does not exist" % ( file) )
		try:
			import ConfigParser
			conf=ConfigParser.ConfigParser()
			conf.read(file)
			if not conf.sections() or not conf.has_section("Project"): raise ConfigParser.ParsingError(file)
			micrographs={}
			sections_sorted=conf.sections()
			sections_sorted.sort()
			
			intfloat=[type(1),type(1.0)]
			for section in sections_sorted:
				if section == "Project":
					attrs=self.__dict__
					attrkeys=attrs.keys()
					attrkeys2=map(string.lower,attrkeys)	# ConfigParser convert the option values to lower case. there is no way to turn this behavior off
					for option in conf.options(section):
						if option in attrkeys2:
							index=attrkeys[attrkeys2.index(option)]
							option=conf.get(section,option).split("#")[0].rstrip()
							if index == "ProcessingRoot":
								import glob
								if glob.glob(os.path.join(option,"*.img")) or glob.glob(os.path.join(option,"*.hed")) or glob.glob(os.path.join(option,"*.map")) or glob.glob(os.path.join(option,"*.mrc")):
									ans=raw_input("Warning: it seems \"ProcessingRoot\" directory (%s) is not dedicated to this processing, it is dangerous and might damage your source data. Are you sure \
										to continue (<cr>-exit, y-continue):" % (option))
									if not (ans=="y" or ans=="Y"): sys.exit(1)
							try:
								v=eval(option)
								if type(v)==type(attrs[index]):
									attrs[index]=v
								elif type(attrs[index])==type(1) and type(v) in intfloat:
									attrs[index]=int(v)
								elif type(attrs[index])==type(1.0) and type(v) in intfloat:
									attrs[index]=float(v)
								elif type(v)!=type(""): raise ValueError("wrong value \"%s\" for Project option \"%s\", it should be type %s like \"%s\"",v,attrkeys[index],type(attrs[index]),attrs[index])
							except:
								if type(option)==type(attrs[index]): attrs[index]=option
								elif type(option)!=type(""): raise ValueError("wrong value \"%s\" for Project option \"%s\", it should be type %s like \"%s\"",option,attrkeys[index],type(attrs[index]),attrs[index])
				elif section[:10] == "micrograph":
					pair=None
					try:
						(dummy,num,pair)=string.split(section)
					except:
						(dummy,num)=string.split(section)
					try:
						num=int(num)
					except:
						raise ValueError("wrong format for micrograph definition header \"[%s]\"" % section)
					if pair not in [None,"pair"]:
						raise ValueError("wrong format for micrograph definition header \"[%s]\"" % section)

					if not micrographs.has_key(num):
						if pair=="pair": raise ValueError("far-from-focus pair micrograph number %d is defined before the close-to-focus micrograph is defined" % num)
						else: micrographs[num]=Micrograph()
					elif pair != "pair": raise ValueError("duplicate definition for micrograph number %d" % num)
					if pair == "pair" and not micrographs[num].pair: 
						micrographs[num].pair=Micrograph()
						cm=micrographs[num].pair
					elif pair == "pair" and micrographs[num].pair:
						raise ValueError("duplicate definition for pair micrograph number %d" % num)
					else:
						cm=micrographs[num]
					attrs=cm.__dict__
					attrkeys=attrs.keys()
					attrkeys2=map(string.lower,attrkeys)
					for option in conf.options(section):
						if option in attrkeys2:
							index=attrkeys[attrkeys2.index(option)]
							option=conf.get(section,option).split("#")[0].rstrip()
							try:
								v=eval(option)
								if type(v)==type(attrs[index]):
									attrs[index]=v
								elif type(attrs[index])==type(1) and type(v) in intfloat:
									attrs[index]=int(v)
								elif type(attrs[index])==type(1.0) and type(v) in intfloat:
									attrs[index]=float(v)
								elif type(v)!=type(""): raise ValueError("wrong value \"%s\" for Micrograph option \"%s\", it should be type %s like \"%s\"",v,attrkeys[index],type(attrs[index]),attrs[index])
							except:
								if type(option)==type(attrs[index]):
									attrs[index]=option
								elif type(option)!=type(""): raise ValueError("wrong value \"%s\" for Micrograph option \"%s\", it should be type %s like \"%s\"",option,attrkeys[index],type(attrs[index]),attrs[index])
			self.Data=[]
			mkeys=micrographs.keys()
			mkeys.sort()
			for m in mkeys:
				self.AddMicrograph(micrographs[m])
			self.ProjectFile=os.path.abspath(file)
		except ConfigParser.ParsingError:
			import EM
			execfile(file,globals(),locals())
			self.__dict__=locals()["proj"].__dict__
			print("Warning: the project file is in an old format, the next time you save this file it will be in the new format.")
		# check for duplicate micrographs
		mgraphs={}
		for i in range(len(self.Data)):
			m=self.Data[i]
			imgfile=os.path.abspath(m.imagefile)
			if mgraphs.has_key(imgfile): raise ValueError("duplicate micrograph found for micrograph %d with image file \"%s\". Please edit the project file \"%s\" to remove the duplication." % (i, m.imagefile, file))
			else: mgraphs[imgfile]=m
			if m.pair:
				imgfile=os.path.abspath(m.pair.imagefile)
				if mgraphs.has_key(imgfile): raise ValueError("duplicate micrograph found for micrograph pair %d with image file \"%s\". Please edit the project file \"%s\" to remove the duplication." % (i, m.pair.imagefile, file))
				else: mgraphs[imgfile]=m.pair
		
	def SaveProjectFile(self,file=None):
		"""Save the Project information into a project file which could be read in by EM.Project.ReadProjectFile() function"""
		if not file: self.ProjectFile=os.path.join(self.ProjectRoot, os.path.basename(self.ProjectFile))		
		else: self.ProjectFile=os.path.abspath(file)
		fp=open(self.ProjectFile,"w")
		fp.write(self.__repr__())
		fp.close()
	def AlignFocalPairs(self, workdir=None, force=0):
		if not workdir: workdir=self.__EMANDir__
		workdir=os.path.abspath(workdir)
		cmd=""
		for m in self.Data:
			if m.pair and type(m.pair) == type(m) and os.path.exists(m.micrograph) and os.path.exists(m.pair.micrograph) and \
			   ( force or not os.path.exists(m.pair.micrograph_aligned)):
				pair_mgraph=os.path.basename(os.path.splitext(m.pair.micrograph)[0])
				mgraph=os.path.basename(os.path.splitext(m.micrograph)[0])
				aligned_micrograph=os.path.join(workdir,pair_mgraph+"-aligned-to-"+mgraph+".mrc")
				m.pair.micrograph_aligned=aligned_micrograph
				alignhuge_log=os.path.splitext(m.pair.micrograph_aligned)[0]+".log"
				if force or not os.path.exists(m.pair.micrograph_aligned) or os.path.getmtime(m.pair.micrograph_aligned)<os.path.getmtime(m.pair.micrograph) or \
				   os.path.getmtime(m.pair.micrograph_aligned)<os.path.getmtime(m.micrograph):
					if not force and m.pair.transform_to_first:
						ccmd="cd %s ; alignhuge %s %s %s transform=%f,%f,%f,%f" % (workdir, m.pair.micrograph, m.micrograph, m.pair.micrograph_aligned, \
								m.pair.transform_to_first[0], m.pair.transform_to_first[1], m.pair.transform_to_first[2], m.pair.transform_to_first[3])
					else:
						ccmd="cd %s ; alignhuge %s %s %s > %s" % (workdir, m.pair.micrograph, m.micrograph, m.pair.micrograph_aligned, alignhuge_log)
					if self.RunPar and self.CPU > 1: cmd=cmd+ccmd+"\n"
					else:
						try:
							if self.Verbose: print ccmd
							os.system(ccmd)
						except: raise RuntimeError("during running command \"%s\"" % ccmd)
		if cmd and self.RunPar and self.CPU > 1:	self.RunParCommands(cmd, jobfile=os.path.join(self.ProjectRoot,"align-focal-pairs.runpar"))
		for m in self.Data:
			if m.pair and type(m.pair) == type(m) and os.path.exists(m.micrograph) and os.path.exists(m.pair.micrograph) and not m.pair.transform_to_first:
				alignhuge_log=os.path.splitext(m.pair.micrograph_aligned)[0]+".log"
				if os.path.exists(alignhuge_log) and os.path.exists(m.pair.micrograph_aligned):
					fp=open(alignhuge_log)
					lines=fp.readlines()
					fp.close()
					if lines[-1][:6]=="Final:":
						[dx,dy,rot]=string.split(string.strip(lines[-1][6:]),",",3)
						m.pair.transform_to_first=[float(dx),float(dy),float(rot)]
					else: raise ValueError("%s does not include correct micrograph transform information" % alignhuge_log)
				else: raise RuntimeError("focal pair micrograph \"%s\" is not aligned to the first micrograph \"%s\"" % (m.pair.micrograph,m.micrograph))
	def BoxWithBoxCoordinates(self, workdir=None, force=0):
		"""Box out particle using known particle coordinates"""
		if not workdir: workdir=self.__EMANDIR__
		cmd=""
		for m in self.Data:
			if os.path.exists(m.boxfile) and os.path.exists(m.micrograph) and (force or (not os.path.exists(m.imagefile) or \
				(os.path.exists(m.imagefile) and os.path.getmtime(m.imagefile)<os.path.getmtime(m.boxfile)))):
				if os.path.exists(m.imagefile):
					os.remove(os.path.splitext(m.imagefile)[0]+".img")
					os.remove(os.path.splitext(m.imagefile)[0]+".hed")
				else:
					m.imagefile=os.path.join(workdir,os.path.splitext(os.path.basename(m.micrograph))[0]+".img")
				tcmd="batchboxer dbbox=%s input=%s output=%s" % (m.boxfile,m.micrograph,m.imagefile)
				if not m.micrograph_positive: tcmd=tcmd+" invert"
				m.image_positive=1
				if self.RunPar and self.CPU>1: cmd=cmd+tcmd+"\n"
				else:
					if self.Verbose: print tcmd
					try:
						os.system(tcmd)
					except: raise RuntimeError("during running command \"%s\"" % (tcmd))
				if m.pair and type(m) == type(m.pair) and os.path.exists(m.pair.micrograph_aligned):
					if os.path.exists(m.pair.imagefile):
						os.remove(os.path.splitext(m.pair.imagefile)[0]+".img")
						os.remove(os.path.splitext(m.pair.imagefile)[0]+".hed")
					else:
						m.pair.imagefile=os.path.join(workdir,os.path.splitext(os.path.basename(m.pair.micrograph_aligned))[0]+".img")
					tcmd="batchboxer dbbox=%s input=%s output=%s" % (m.boxfile,m.pair.micrograph_aligned,m.pair.imagefile)
					if not m.pair.micrograph_positive: tcmd=tcmd+" invert"
					m.pair.image_positive=1
					if self.RunPar and self.CPU>1: cmd=cmd+tcmd+"\n"
					else:
						if self.Verbose: print tcmd
						try:
							os.system(tcmd)
						except: raise RuntimeError("during running command \"%s\"" % (tcmd))
		if cmd: self.RunParCommands(cmd,jobfile=os.path.join(self.ProjectRoot,"boxing-with-box-coordinates.runpar"))
	def AutoBoxing(self):
		cmd=""
		for m in self.Data:
			mgraph=""
			if m.micrograph_positive==None: m.micrograph_positive=self.PositiveContrast
			if m.pair and type(m.pair)==type(m):
				if m.pair.micrograph_positive==None: m.pair.micrograph_positive=self.PositiveContrast
				if m.pair.micrograph_positive!=m.micrograph_positive:
					raise ValueError("does not handle different contrast for focal micrographs yet:\n%s" % m)
				if os.path.exists(m.pair.micrograph_merged):
					mgraph=m.pair.micrograph_merged
				elif os.path.exists(m.pair.micrograph_aligned):
					mgraph=m.pair.micrograph_aligned
			elif os.path.exists(m.micrograph):
				mgraph=m.micrograph
			if not mgraph:
				if m.pair and type(m.pair)==type(m) and not os.path.exists(m.pair.imagefile):
					print "Warning: cannot find enough information to autobox from-from-focus micrograph\n%s, only the close-to-focus micrograph is used from now on" % (m.pair)
					m.pair=None
				elif not (m.imagefile and os.path.exists(m.imagefile)):
					raise ValueError("cannot find enough information to autobox micrograph:\n%s" % (m))
				else:
					continue
			image_file=os.path.join(self.__EMANDir__,os.path.splitext(os.path.basename(mgraph))[0]+"-autobox.img")
			box_file=os.path.join(self.__EMANDir__,os.path.splitext(os.path.basename(mgraph))[0]+"-autobox.box")
			if mgraph!=m.micrograph:
				if not m.boxfile: m.boxfile=box_file
				if not m.pair.boxfile: m.pair.boxfile=box_file
				if not m.pair.imagefile: m.pair.imagefile=image_file
				if not m.imagefile:
					image_file2=os.path.join(self.__EMANDir__,os.path.splitext(os.path.basename(m.micrograph))[0]+"-autobox.img")
					m.imagefile=image_file2
				if os.path.exists(m.imagefile) and os.path.getmtime(m.imagefile)>os.path.getmtime(m.micrograph) and \
				   os.path.exists(m.pair.imagefile) and os.path.getmtime(m.pair.imagefile)>os.path.getmtime(mgraph) \
				   and (not os.path.exists(m.boxfile) or (os.path.exists(m.boxfile) and (os.path.getmtime(m.boxfile) < \
				   os.path.getmtime(m.imagefile) or os.path.getmtime(m.boxfile)<os.path.getmtime(m.imagefile)))):
					if m.image_positive==None: m.image_positive=1
					if m.pair.image_positive==None: m.pair.image_positive=1
					continue
			else:
				if not m.boxfile: m.boxfile=box_file
				if not m.imagefile: m.imagefile=image_file	
				if os.path.exists(m.imagefile) and os.path.getmtime(m.imagefile)>os.path.getmtime(mgraph) \
				   and (not os.path.exists(m.boxfile) or (os.path.exists(m.boxfile) and os.path.getmtime(m.boxfile) < \
					os.path.getmtime(m.imagefile))):
					if m.image_positive==None: m.image_positive=1
					continue
			m.image_positive=1
			if m.apix: apix=m.apix
			else: apix=self.PixelSize
			if not m.micrograph_positive: protein_is_white=0
			else: protein_is_white=1
			if os.path.exists(m.boxfile) and ((os.path.exists(m.imagefile) and \
			   os.path.getmtime(m.boxfile)>os.path.getmtime(m.imagefile)) or (m.pair and type(m.pair)==type(m) \
				and os.path.exists(m.pair.imagefile) and os.path.getmtime(m.boxfile) \
				>os.path.getmtime(m.pair.imagefile))):
				if os.path.exists(m.imagefile[:-4]+".img"): os.remove(m.imagefile[:-4]+".img")
				if os.path.exists(m.imagefile[:-4]+".hed"): os.remove(m.imagefile[:-4]+".hed")
				tcmd=tcmd+"; batchboxer dbbox=%s input=%s output=%s" % (m.boxfile,mgraph,image_file)
				if not protein_is_white: tcmd=tcmd+" invert"
			else:
				tcmd=AutoBoxing(micrograph_file=mgraph,particle_file=image_file,box_file=box_file,radius=self.ParticleSize/apix/2.,imagesize=self.ImageSize,protein_is_white=protein_is_white,verbose=self.Verbose,cmdonly=1)
			if mgraph!=m.micrograph and os.path.exists(m.micrograph):
				m.pair.image_positive=1
				if os.path.exists(m.imagefile[:-4]+".img"): os.remove(m.imagefile[:-4]+".img")
				if os.path.exists(m.imagefile[:-4]+".hed"): os.remove(m.imagefile[:-4]+".hed")
				tcmd=tcmd+"; batchboxer dbbox=%s input=%s output=%s" % (m.boxfile,m.micrograph,m.imagefile)
				if not protein_is_white: tcmd=tcmd+" invert"
			if tcmd:
				if self.RunPar and self.CPU>1: cmd=cmd+tcmd+"\n"
				else:
					if self.Verbose: print tcmd
					try:
						os.system(tcmd)
					except: raise RuntimeError("during running command \"%s\"" % (tcmd))
		if cmd: self.RunParCommands(cmd,jobfile=os.path.join(self.ProjectRoot,"auto-boxing.runpar"))
	def AutoFitCTF(self):
		""" automatically fit the CTF parameters for each micrograph """
		if self.RunPar and self.CPU>1:
			jobs=""
			for m in self.Data:
				tjob=m.AutoFitCTF(self,fixedac=self.AmpContrast)
				if tjob: jobs=jobs+"\n"+tjob
				if m.pair and type(m.pair)==type(m) and m.pair.imagefile and os.path.exists(m.pair.imagefile): 
					tjob=m.pair.AutoFitCTF(self,fixedac=self.AmpContrast)
					if tjob: jobs=jobs+"\n"+tjob
			self.RunParCommands(jobs)
			for m in self.Data: 
				m.CTFParameters(self,fixedac=self.AmpContrast)
				if m.pair and type(m.pair)==type(m) and m.pair.imagefile and os.path.exists(m.pair.imagefile):
					m.pair.CTFParameters(self,fixedac=self.AmpContrast)
		else:
			for m in self.Data: 
				#m.PrepareDirsFiles(self,cycle=cycle)
				m.CTFParameters(self,fixedac=self.AmpContrast)
				if m.pair and type(m.pair)==type(m) and m.pair.imagefile and os.path.exists(m.pair.imagefile):
					m.pair.CTFParameters(self,fixedac=self.AmpContrast)
	def MergeFocalPairs(self):
		cmd=""
		for m in self.Data:
			if m.pair and type(m.pair) == type(m) and os.path.exists(m.micrograph) and os.path.exists(m.pair.micrograph_aligned) and \
			   not os.path.exists(m.pair.micrograph_merged):
				pair_mgraph=os.path.basename(os.path.splitext(m.pair.micrograph)[0])
				mgraph=os.path.basename(os.path.splitext(m.micrograph)[0])
				merged_micrograph=os.path.join(self.__EMANDir__,pair_mgraph+"-merged-to-"+mgraph+".mrc")
				if not m.pair.micrograph_merged: m.pair.micrograph_merged=merged_micrograph
				if not os.path.exists(m.pair.micrograph_merged) or os.path.getmtime(m.pair.micrograph_merged)<os.path.getmtime(m.pair.micrograph_aligned) or \
				   os.path.getmtime(m.pair.micrograph_merged)<os.path.getmtime(m.micrograph):					
					ccmd="cd %s ; focalpair %s %s %s im1parm=%s im2parm=%s noampc" % \
						  (self.__EMANDir__, m.pair.micrograph, m.micrograph, m.pair.micrograph_merged, \
						   m.pair.__ctfstr__,m.__ctfstr__)
					if os.path.exists(self.SFFile): ccmd=ccmd+" sffile=%s" % (self.SFFile) 
					if self.RunPar and self.CPU > 1: cmd=cmd+ccmd+"\n"
					else:
						try:
							if self.Verbose: print ccmd
							os.system(ccmd)
						except: raise RuntimeError("during running command \"%s\"" % ccmd)
		if cmd and self.RunPar and self.CPU > 1:	self.RunParCommands(cmd, jobfile=os.path.join(self.ProjectRoot,"align-focal-pairs.runpar"))
	def SetupRefineCycles(self):
		"""Set up the resolution steps for refinement. The step size will gradually decrease from 5 pixels (low resolution) to 2 pixels (high resolution)"""
		if len(self.ResRange)!=2 or self.ResRange[0]<=self.ResRange[1] or self.ResRange[1] < self.PixelSize*2.0/0.9:
			print "\"ResRange="+`self.ResRange`+"\" is not correct, will be changed to \"[30,%.2f]\"" % ( self.PixelSize*2./0.9 )
			self.ResRange=[30,self.PixelSize*2./0.9]	# it is not good idea to refine too close to the edge of FFT
		if not len(self.MRCRefine): # won't override if it is already defined by user
			if self.WaveCycles<=0:
				self.WaveCycles=0
				self.MRCRefine.append(self.ResRange[0])
			else:
				for i in range(self.WaveCycles):
					self.MRCRefine.append(self.ResRange[0]-0.02*i)
			invs=float(self.PixelSize*self.ImageSize)
			pstart=int(invs/float(self.ResRange[0])+0.5)
			pend=int(invs/float(self.ResRange[1])+0.5)
			if not (len(self.RefineCycleSize)==2 and self.RefineCycleSize[0]>=self.RefineCycleSize[-1]):
				self.RefineCycleSize=[0,0]
				self.RefineCycleSize[0]=int(invs/25.+0.5)-int(invs/30.+0.5)
				self.RefineCycleSize[-1]=self.RefineCycleSize[0]/2
			startstep=self.RefineCycleSize[0]	# start with more pixels at low resolution
			stopstep=self.RefineCycleSize[-1]	# end with fewer pixels at high resolution
			cp=pstart
			while cp < pend:
				cstep=float(startstep)-float(cp-pstart)/float(pend-pstart)*float(startstep-stopstep)
				cp=cp+int(cstep+0.5)
				self.MRCRefine.append(invs/float(cp))
			if hasattr(self,"RefineMore") and self.RefineMore>0 and self.RefineMore < self.ResRange[-1]:
				pend=int(invs/float(self.RefineMore)+0.5)
				while cp < pend:
					cp=cp+int(cstep+0.5)
					self.MRCRefine.append(invs/float(cp))
			if self.Verbose:
				print "The %d refinement cycles are: %s" % (len(self.MRCRefine), self.MRCRefine)
		return self.MRCRefine
	def CreateCTFCurveParaFile(self,usefixedB=0):
		"""Create the CTF files needed for "WaveOrt", "refineAll" and "reconstructParallel" for CTF correction""" 
		# not sure why tmpnam() is not allowd, even though it is in the document
		#dbfile=os.tmpnam()	# create the database file in system tmp dir
		dbfile=os.path.join(self.ProjectRoot,"MRC-CTF-DB.dat")		# create the database file in system tmp dir
		if not os.path.exists(dbfile):
			dbfp=open(dbfile,'w')
			for m in self.Data:
				#mdir=os.path.join(self.ProjectRoot,m.__mrcworkdir__)
				#print m.__mrcworkdir__, mdir
				dbfp.write("%s:\t%f\t%f\n" % (m.__mrcworkdir__,m.defocus,m.B))
				p=m.GetFocalPair()
				if p:
					#pdir=os.path.join(self.ProjectRoot,p.__mrcworkdir__)
					dbfp.write("%s:\t%f\t%f\n" % (p.__mrcworkdir__,p.defocus,p.B))
			dbfp.close()
		cycle2=len(self.MRCRefine)-1	#does not hurt to include more 
		cmd=os.popen("ctfForAll %s %s" % (dbfile,self.__MRCCTFFile__),'w')
		cmd.write("%d\n" % (self.Voltage))
		cmd.write("%f\n" % (self.Cs))
		cmd.write("%f\n" % (self.AmpContrast))
		if usefixedB>0: 
			cmd.write("y\n%f\n" % (usefixedB))
		else:
			cmd.write("n\n0\n")
		cmd.write("1\n")		# not sure if 1 or 1000 is used
		cmd.write("%f\n" % (self.__CPixelSize__))
		cmd.write("%d\n0.1\n1\n\n" % (self.__CImageSize__))
		cmd.write("%d\n" % (int(0.5+self.__CImageSize__*self.__CPixelSize__/self.MRCRefine[cycle2])))
		cmd.close()

		# this should be replaced though
		for m in self.Data:
			if not os.path.exists(os.path.join(m.__emanworkdir__,m.__ctffile__)): # setup the ctf parameters
				ctffp=open(os.path.join(m.__emanworkdir__,m.__ctffile__),'w')
				ctffp.write("%s\t-%f,%f,1.0,%f,0,0,0,0,%f,%f,%f,%s\n" % (m.imagefile,m.defocus,m.B,self.AmpContrast,self.Voltage,self.Cs,self.__CPixelSize__,self.SFFile))
				ctffp.close()
			p=m.GetFocalPair()
			if p and not os.path.exists(os.path.join(p.__emanworkdir__,p.__ctffile__)): # setup the ctf parameters
				ctffp=open(os.path.join(p.__emanworkdir__,p.__ctffile__),'w')
				ctffp.write("%s\t-%f,%f,1.0,%f,0,0,0,0,%f,%f,%f,%s\n" % (p.imagefile,p.defocus,p.B,self.AmpContrast,self.Voltage,self.Cs,self.__CPixelSize__,self.SFFile))
				ctffp.close()
	def TestCTFParameters(self):
		"""Test the the performance of automatic CTF fitting routine by comparing to the hand fitted values"""
		self.SetupRefineCycles()
		mean_dB=0
		sigma_dB=0
		mean_ddefocus=0
		sigma_ddefocus=0
		count=0
		mgraphs=[]
		for m in self.Data:
			mgraphs.append(m)
			p=m.GetFocalPair()
			if p: mgraphs.append(p)
		ctffile=open("ctf-compare.txt",'w')
		for m in mgraphs:
			B0=m.B
			defocus0=m.defocus
			m.PrepareDirsFiles(self)
			newctf=m.CTFParameters(self,replace=2, fixedac=0)
			if newctf:
				dB=newctf["B"]-B0
				ddefocus=newctf["defocus"]-defocus0
				mean_dB=mean_dB+dB
				sigma_dB=sigma_dB+dB*dB
				mean_ddefocus=mean_ddefocus+ddefocus
				sigma_ddefocus=sigma_ddefocus+ddefocus
				count=count + 1
				str="Micrograph %s: original B = %f\tdZ = %f\tnew B = %f\tdZ = %f\tdB = %f\tddZ = %f\n" % (os.path.basename(os.path.splitext(m.imagefile)[0]), \
					B0,defocus0,newctf["B"],newctf["defocus"],dB,ddefocus)
				print str
				ctffile.write(str)
		mean_dB=mean_dB/count
		mean_ddefocus=mean_ddefocus/count
		sigma_dB=math.sqrt(sigma_dB/count-mean_dB*mean_dB)
		sigma_ddefocus=math.sqrt(sigma_ddefocus/count-mean_ddefocus*mean_ddefocus)
		str="Mean dB = %f sigma = %f\tMean ddZ = %f sigma = %f\n" % (mean_dB,sigma_dB,mean_ddefocus,sigma_ddefocus)
		print str
		ctffile.write(str)
		ctffile.close()
	def Split(self,num=2):
		"""Split the Project into "num" of even parts including the individual micrographs. Return the list of daughter Projects"""
		if num<=1: return [self]
		newprojs=[None]*num
		for i in range(num):
			newprojs[i]=copy.deepcopy(self)	# must do deep copy
			newprojs[i].ProjectRoot=os.path.join(self.ProjectRoot,"%d-splits/set-%d" % (num,i))
		newmgraph=[None]*len(self.Data)
		for i in range(len(self.Data)):
			newmgraph[i]=self.Data[i].Split(num,self.Verbose)
		for set in range(num):
			for i in range(len(self.Data)):
				newprojs[set].Data[i]=newmgraph[i][set]
		return newprojs
	def CopyProject(self,OldProjFile,NewProjFile='NewProject.prj',NewSrcImgDir='./SourceImages',NewRootDir=''):
		"""	Copy the project (source images and the processing directories) into a new location and output a new Project file accordingly.
			This is used when home directory is moved or a new user takes over the project
		"""		
		try:	
			# now modify the Project file
			if not os.path.exists(OldProjFile):
				print "cannot open the old project root file \"%s\", ignored" % (OldProjFile)
				return 0
			
			from shutil import copy2, copytree
			if not os.path.exists(NewSrcImgDir): os.makedirs(NewSrcImgDir)
			
			rep=[[self.StartModel,os.path.join(os.path.abspath(NewSrcImgDir),os.path.basename(self.StartModel))]]
			for m in self.Data:
				mname=os.path.basename(m.imagefile)
				rep.append([m.imagefile,os.path.abspath(os.path.join(NewSrcImgDir,mname))])
				if m.pair and type(m.pair) == type(m):
					mname=os.path.basename(m.pair.imagefile)
					rep.append([m.pair.imagefile,os.path.abspath(os.path.join(NewSrcImgDir,mname))])

			if NewRootDir: rep.append([self.ProjectRoot,os.path.abspath(NewRootDir)])
			fp=open(OldProjFile)
			lines=fp.readlines()
			fp.close()
			for i in range(len(lines)):
				for j in range(len(rep)):
					lines[i]=string.replace(lines[i],rep[j][0],rep[j][1])
			fp=open(os.path.join(NewSrcImgDir,NewProjFile),'w')
			fp.writelines(lines)
			fp.close()

			copy2(self.StartModel,NewSrcImgDir) # copy the start model map
			
			for m in self.Data: # copy the source images 
				hedfile=os.path.splitext(m.imagefile)[0]+'.hed'
				copy2(m.imagefile,NewSrcImgDir)
				copy2(hedfile,NewSrcImgDir)
				if m.pair and type(m.pair) == type(m):
					hedfile=os.path.splitext(m.pair.imagefile)[0]+'.hed'
					copy2(m.pair.imagefile,NewSrcImgDir)
					copy2(hedfile,NewSrcImgDir)
			
			if not NewRootDir: return	# sometimes we don't want to copy the processing files	
			try:	
				if not os.path.exists(NewRootDir): os.makedirs(NewRootDir)
				#os.system("cp -rf %s %s" % (self.ProjectRoot, NewRootDir))
				copytree(self.ProjectRoot, NewRootDir, 1) # copy the RootDir
			except KeyboardInterrupt:
				return 0
			except:
				print "Error: in copying the project root dir, make sure the new root dir \"%s\" does exist" % (os.path.abspath(NewRootDir))
				return 0
			for m in self.Data: # delete the wrong link in new micrograph dir
				mname=os.path.basename(os.path.splitext(m.imagefile)[0])
				newmdir=os.path.join(os.path.abspath(NewRootDir),mname,'eman-workdir')
				imgfile=os.path.join(newmdir,mname+'.img')
				hedfile=os.path.join(newmdir,mname+'.hed')
				if os.path.islink(imgfile): os.remove(imgfile)
				if os.path.islink(hedfile): os.remove(hedfile)
				
			if self.ProjectRoot[-1]=="/" and NewRootDir[-1]!="/": NewRootDir=NewRootDir+"/"

			def replaceroot(arg,subdir,names):
				from glob import glob
				files=glob(os.path.join(subdir,arg[0]))
				for line in fileinput.input(files,inplace=1):
					string.replace(line, arg[1],arg[2])
			os.path.walk(NewRootDir,replaceroot,["*Angstrom*.dat",self.ProjectRoot,NewRootDir])
			return 1
		except KeyboardInterrupt:
			return 0
	def BuildInitialModel(self, relax=0):
		"""	Build the initial model from the micrographs which have the first CTF peak closest to resolution defined by "self.MRCRefine[0]"
			This is done using "proc2d", "cenalignint", "ortAll", "buildTempalte", "globalRefine" and "reconstructParallel"
		"""
		# first find the best micrograph that has the first CTF peak at the resolution MRCRefine[0]
		#data2=copy.deepcopy(self.Data)
		data2=[]
		for m in self.Data: 
			m.CtfFirstPeakZero(self)
			if m.firstzero<=self.MRCRefine[0] and m.firstpeak>=self.MRCRefine[0]:
				data2.append(copy.deepcopy(m))
				#data2[-1].peakdiff=math.fabs(self.MRCRefine[0]-data2[-1].firstpeak)
			p=m.GetFocalPair()
			if p:
				p.CtfFirstPeakZero(self)
				if p.firstzero<=self.MRCRefine[0] and p.firstpeak>=self.MRCRefine[0]:
					data2.append(copy.deepcopy(p))
					#data2[-1].peakdiff=math.fabs(self.MRCRefine[0]-data2[-1].firstpeak)
		if not data2: raise ValueError("cannot find suitable micrographs in right focus range to build initial model")
		lowestres=0
		for m in data2:
			if m.firstzero>lowestres: lowestres=m.firstzero
		res=int(self.ImageSize*self.PixelSize/lowestres)
		#data2.sort(lambda m1, m2: cmp(m1.peakdiff,m2.peakdiff))
		data2.sort(lambda m1, m2: cmp(m2.defocus,m1.defocus))
		if self.Verbose:
			for m in range(len(data2)):
				print "Micrograph %2d: %s, defocus %f um, first peak %f angstrom, first zero %.4f angstrom" % (m,data2[m].imagefile,data2[m].defocus,data2[m].firstpeak,data2[m].firstzero)
		#check for initial model work directory
		initdir=os.path.join(self.ProjectRoot,"initmodel-dir")
		if not os.path.exists(initdir): 
			os.makedirs(initdir)
		os.chdir(initdir)
		
		if not os.path.exists("definedort.dat"):		# has not succesfully build template and eliminated bad orientations yet
			cmd="rm -rf start.* avg.*"
			if self.Verbose: print cmd
			#os.system(cmd)
			imgnum=0
			goodlognum=0
			goodimgnum=0
			eliminatestart=0
			angdiff=self.MRCRefine[0]*2./self.ParticleSize*180./math.pi

			for m in data2: 
				try:
					imgnum=ImageInfo(filename="start.img")["num"]
				except:
					imgnum=0
				
				imagesize=ImageInfo(filename=m.imagefile)["nx"]
				
				#mgraph=os.path.basename(os.path.splitext(m.imagefile)[0])
				#cemanworkdir=os.path.join(self.ProjectRoot,mgraph,"eman-workdir")
				imgbase=os.path.basename(os.path.splitext(m.imagefile)[0])
				imgfile=os.path.join(m.__emanworkdir__,imgbase+".img")
				if not os.path.exists(imgfile): imgfile=m.imagefile

				#cmd="proc2d "+m.imagefile+" tmp.img lp="+`int(imagesize*m.apix/lowestres)`
				#cmd=cmd+"; cenalignint tmp.img maxshift="+`int((imagesize-self.ParticleSize/m.apix)/2.0)`
				cmd="cenalignint "+imgfile+" maxshift="+`int((imagesize-self.ParticleSize/m.apix)/2.0)`
				cmd=cmd+"; proc2d ali.img start.img ; rm bad.* ali.*"
				if self.Verbose: print cmd
				os.system(cmd)
				try:
					imgnum1=ImageInfo(filename="start.img")["num"]
				except:
					imgnum1=0
					continue
				if imgnum1==imgnum: continue

				#cmd="starticos start.hed "+`int(imgnum*0.1)`
				#os.system(cmd)
			
				#res=int(self.ImageSize*self.PixelSize/lowestres)
				#OrtAll("start.img",startimg=imgnum,mask2=int((self.ImageSize+self.ParticleSize/self.PixelSize)/4.),res2=res, verbose=self.Verbose)
				OrtAll("start.img",self.__CPixelSize__,startimg=imgnum,mask1=int(self.ParticleSize*self.RefineMaskFactor[0]/self.PixelSize/2.),mask2=int(self.ParticleSize*self.RefineMaskFactor[-1]/self.PixelSize/2.),res2=res, verbose=self.Verbose)

				if relax and m is data2[-1]:
					goodlognum=BuildTemplate("start.img",height=4.,width=12,angdiff=angdiff,phase=60,res2=res,min_num_particles=3,relax=relax,verbose=self.Verbose)
				else:
					goodlognum=BuildTemplate("start.img",height=4.,width=12,angdiff=angdiff,phase=60,res2=res,min_num_particles=3,verbose=self.Verbose)
				if goodlognum>=3:
					goodimgnum=EliminateOrt("start.img",startimg=eliminatestart,res2=res,verbose=self.Verbose)
					eliminatestart=imgnum1
					if goodimgnum>=10: break
			if not goodlognum:
				raise ValueError("No particle is good as template for building initial model, please provide more micrographs")
			elif not goodimgnum:
				raise ValueError("No particle is good for building initial model, please provide more micrographs")
		if not os.path.exists("globalRefined.dat"):		# has not succcessfully global refined orientations
			#now run globalRefine
			GlobalRefine("start.img", step=self.MRCRefine[0]*2./self.ParticleSize*180./math.pi*0.5, res2=res, cpu=self.CPU, verbose=self.Verbose)
		
		initmodel=string.joinfields(string.split(self.Sample),'-')+"-init-model.mrc"
		if not os.path.exists(initmodel):
			#now reconstruct the initial model
			diameter=int(int(self.ParticleSize*self.RefineMaskFactor[-1]/self.PixelSize+0.5)/2)*2+1
			if self.MapSize: diameter=self.MapSize

			if self.CPU>res/10: cpu1=res/10	# temporary solution
			else: cpu1=self.CPU
			if self.CPU>self.MapSize/10: cpu2=self.MapSize/10
			else: cpu2=self.CPU
			if cpu2<2: cpu2=2	# very stupid, but I have to do this since the emicofb step of reconstruction code has to use at least 2 cpus

			Reconstruct("globalRefined.dat",diameter=diameter,res=res,memo="initial-model-truncated-at-%.2fAngstrom" % lowestres,cpu1=cpu1,cpu2=cpu2,verbose=self.Verbose)
			#if os.path.exists(initmodel): os.rename(initmodel,initmodel+".old")
			if os.path.exists("icos5f.mrc"):
				os.rename("icos5f.mrc",initmodel)
				if self.Mail: 
					import tempfile
					from socket import gethostname, gethostbyaddr
					images=[]
					if os.path.exists(initmodel): 
						image=tempfile.mktemp(".png")
						try:
							cmd="map2img %s %s" % (initmodel, image)
							if self.Verbose: print cmd
							os.system(cmd)
							if os.path.exists(image): images.append(image)
						except:
							pass
					subject="%s: finished building initial 3D model" % (self.Sample)
					msg="The initial 3D model is generated and saved as \"%s\" on computer \"%s\"" % (os.path.abspath(initmodel), gethostbyaddr(gethostname())[0])
					if images: msg=msg+"\n\nThe central section of the initial 3D model \"%s\" is:" % (os.path.abspath(initmodel))
					SendMail(to=self.Mail,subject=subject,msg=msg,images=images)
					for image in images: 
						if os.path.exists(image): os.remove(image)                 
				
		if  os.path.exists(initmodel):
			self.StartModel=os.path.abspath(initmodel)
			self.SMCTFCorrect=0
			self.SMResolution=self.MRCRefine[0]+0.2
			self.SaveProjectFile(os.path.join(self.ProjectRoot,os.path.basename(self.ProjectFile)))
		
			return self.StartModel
		else:
			return None	
	def PrepareDirectories(self):
		"""Make sure the images exists and contain correct information. Also setup correct directories and neccessary parameters"""
		try:
			 os.mkdir(self.ProjectRoot)
		except:
			pass
		self.__MRC3DDir__	 =	os.path.join(self.ProjectRoot,'mrc-3d-dir')
		self.__MRCProjDir__	 =	os.path.join(self.ProjectRoot,'mrc-proj-dir')
		self.__WAVEProjDir__ = 	os.path.join(self.ProjectRoot,'wave-proj-dir')
		self.__EMANDir__	 =	os.path.join(self.ProjectRoot,'eman-dir')
		if not os.path.exists(self.__MRC3DDir__):
			os.makedirs(self.__MRC3DDir__)
		if not os.path.exists(self.__MRCProjDir__):
			os.makedirs(self.__MRCProjDir__)
		if not os.path.exists(self.__WAVEProjDir__):
			os.makedirs(self.__WAVEProjDir__)
		if not os.path.exists(self.__EMANDir__):
			os.makedirs(self.__EMANDir__)		
		for m in self.Data: m.PrepareDirectories(self)		
	def SetupParameters(self, cycle=0):
		if not len(self.MRCRefine): self.SetupRefineCycles()
		if cycle==0 and self.AddExtraB:
			for m in self.Data:
				m.B=m.B+self.AddExtraB
				if m.pair and type(m.pair)==type(m): m.pair.B=m.pair.B+self.AddExtraB
		
		if cycle==0 and ( (len(self.DefocusRange)==2 and self.DefocusRange[1]>self.DefocusRange[0]) or (len(self.BfactorRange)==2 and self.BfactorRange[1]>self.BfactorRange[0]) ):
			data2=self.Data
			self.Data=[]
			for m in data2:
				if ( len(self.DefocusRange)==2 and m.defocus>=self.DefocusRange[0] and m.defocus<=self.DefocusRange[1] and len(self.BfactorRange)==2 and m.B>=self.BfactorRange[0] and m.B<=self.BfactorRange[1] ) or \
				   ( len(self.DefocusRange)!=2 and len(self.BfactorRange)==2 and m.B>=self.BfactorRange[0] and m.B<=self.BfactorRange[1] ) or \
				   ( len(self.BfactorRange)!=2 and len(self.DefocusRange)==2 and m.defocus>=self.DefocusRange[0] and m.defocus<=self.DefocusRange[1] ) :
					self.Data.append(m)

		if cycle==0 and self.IgnoreFocalPair:
			for m in self.Data:
				if hasattr(m,'pair'): m.pair=None
				
		if not self.SMPixelSize: self.SMPixelSize=self.PixelSize

		if self.MultSampling:
			shrunk=self.ShrinkFactor(cycle)
			self.__CPixelSize__=self.PixelSize*shrunk
			self.__CImageSize__=int(self.ImageSize/shrunk)/4*4
			if self.MapSize: self.__CMapSize__=self.MapSize/shrunk/2*2+1
		else:
			self.__CPixelSize__=self.PixelSize
			self.__CImageSize__=self.ImageSize
			self.__CMapSize__=self.MapSize
			
		if self.__DeepTTestCycle__<0:
			if self.DeepTTestRes>self.MRCRefine[0] and self.DeepTTestRes<self.MRCRefine[-1]:
				self.__DeepTTestCycle__=len(self.MRCRefine)-1
			else:
				refine2=copy.copy(self.MRCRefine)
				for i in range(len(refine2)): refine2[i]=math.fabs(refine2[i]-self.DeepTTestRes)
				self.__DeepTTestCycle__=refine2.index(min(refine2))
		if not self.RefineMaskFactor or self.RefineMaskFactor[0]<self.RefineMaskFactor[-1]: self.RefineMaskFactor=[1.25,1.05]
		self.__CMaskRadius__=self.ParticleSize*(self.RefineMaskFactor[-1]+(1.0-float(cycle)/float(len(self.MRCRefine)-1))*(self.RefineMaskFactor[0]-self.RefineMaskFactor[-1]))/self.__CPixelSize__/2.0
	def PrepareParticleImages(self, cycle=0):
		"""Prepare the working copy of raw particle images""" 		
		if cycle==0 and not self.IgnoreFocalPair: self.AlignFocalPairs()
		if cycle==0: self.AutoBoxing()		
		if cycle==0: self.SortMicrograph(var="imagenum") 
		if self.RunPar and self.CPU>1:
			jobs=""
			for m in self.Data:
				tjob=m.PrepareWorkingImages(self,cycle=cycle)
				if tjob: jobs=jobs+"\n"+tjob
			self.RunParCommands(jobs)
		else:
			for m in self.Data: 
				m.PrepareWorkingImages(self,cycle=cycle)

		if self.CTFCorrect:
			if cycle==0:
				self.AutoFitCTF()
				self.CreateCTFCurveParaFile()
			elif self.MultSampling:
				shrunk=self.ShrinkFactor(cycle)
				if cycle: lshrunk=self.ShrinkFactor(cycle-1)
				else: lshrunk=shrunk
				if lshrunk!=shrunk: self.CreateCTFCurveParaFile()
		if cycle==0: self.SaveProjectFile()
	def PrepareProjectionImages(self, cycle=0):
		#check if the starting model exists, if not try to build it once
		if not self.StartModel or not os.path.exists(self.StartModel):
			self.StartModel=self.BuildInitialModel(relax=1.5)
		if not self.StartModel or not os.path.exists(self.StartModel):
			raise IOError("cannot find start initial 3D model map '"+self.StartModel+"'")

		#if self.Processed(cycle=cycle):
		#	if self.Verbose: print "The whole refinement is already done for cycle of (%d of %d) %.2f Angstrom resolution, skipped" % (cycle,len(self.MRCRefine),self.MRCRefine[cycle])
		#	return None

		os.chdir(self.__MRC3DDir__)
		starttype=os.path.splitext(self.StartModel)[1]
		if not os.path.exists("icos5f-start-model"+starttype): os.symlink(self.StartModel,"icos5f-start-model"+starttype)

		if cycle==0 or self.SMResolution<=self.MRCRefine[cycle]: mrc3dmap=self.StartModel
		else: mrc3dmap = os.path.abspath("icos5f-refined-to-%.2fAngstrom.mrc" % (self.MRCRefine[cycle-1]))
		if not os.path.exists(mrc3dmap): raise ValueError("Cycle %d of %d: cannot find the correct initial model \"%s\", exit..." % ( cycle, len(self.MRCRefine), mrc3dmap))

		eman3dmap = os.path.join(self.__MRC3DDir__,"icos5f-sym-thresh.mrc")
		eman3dmaskedmap= os.path.join(self.__MRC3DDir__,"icos5f-masked.mrc")
		waveproj3dmap=mrc3dmap
		mrcproj3dmap=mrc3dmap

		if (cycle==0 and self.SMPixelSize != self.__CPixelSize__):
			scale=self.SMPixelSize/self.__CPixelSize__
			apix=self.SMPixelSize
		else:
			scale=1.0
			apix=self.__CPixelSize__
		
		if os.path.exists(mrc3dmap) and (not os.path.exists(eman3dmap) or os.path.getmtime(eman3dmap)<os.path.getmtime(mrc3dmap)):
			info=ImageInfo(mrc3dmap)
			cmd=""
			if os.path.exists(eman3dmap): os.remove(eman3dmap)
			if os.path.exists(eman3dmaskedmap): os.remove(eman3dmaskedmap)
			if self.Symmetrize or (cycle==0 and self.SMPixelSize != self.__CPixelSize__ and os.path.samefile(mrc3dmap,self.StartModel)):
				cpus=self.CPU
				if info['nz']<self.CPU: cpus=info['nz']
				#cpus=1
				tmpmap="temp-icos5f-sym-thresh.img"
				tmpmap2="temp-icos5f-sym-thresh.hed"
				if os.path.exists(tmpmap): os.remove(tmpmap)
				if os.path.exists(tmpmap2): os.remove(tmpmap2)
				cmd=cmd+"symmetrizeP.py "+mrc3dmap+" "+eman3dmap+" --mrc --cpu="+`cpus`+" --sym=icos"
				if (self.CTFCorrect and cycle>0) or (self.SMCTFCorrect and cycle==0): cmd=cmd+" --thresh=0,p"
				elif (not self.CTFCorrect and cycle>0) or (not self.SMCTFCorrect and cycle==0): cmd=cmd+" --thresh=0,n"
				#if self.Symmetrize: cmd=cmd+" --sym=icos"
				#else: cmd=cmd+" --sym=c1"
				if self.Verbose: cmd=cmd+ " --verbose"
				#cmd=cmd+"; proc3d %s %s ;rm %s %s" % (tmpmap,eman3dmap,tmpmap,tmpmap2)
			elif not os.path.samefile(mrc3dmap,eman3dmap): 
				#cmd=cmd+"proc3d %s %s" % (mrc3dmap,eman3dmap)
				cmd=cmd+"cp %s %s" % (mrc3dmap,eman3dmap)
			waveproj3dmap=eman3dmap
			mrcproj3dmap=eman3dmap
			if self.WaveInMask or self.CCMLInMask:
				cmd=cmd+"; icosmask "+eman3dmap+" "+eman3dmaskedmap+" apix="+`apix`+" inmask="+`self.InnerRadius`+","+`self.ICFactor*self.InnerRadius`+" outmask="+`self.ParticleSize*self.RefineMaskFactor[-1]/2.0`+","+`self.ICFactor*self.ParticleSize*self.RefineMaskFactor[-1]/2`+" soft="+`self.IMaskSlope` 
				if self.WaveInMask: waveproj3dmap=eman3dmaskedmap
				elif self.CCMLInMask: mrcproj3dmap=eman3dmaskedmap
			if self.MolMass:
				if info['nx']==info['ny'] and info['nx']==info['nz']: factor=1.0
				else: factor=2.0
				cmd=cmd+"; volume %s %f set=%f; thresh %s %s thresh=1" % (eman3dmap, apix, self.MolMass/factor, eman3dmap, eman3dmap)
				if self.WaveInMask or self.CCMLInMask: 
					cmd=cmd+"; volume %s %f set=%f; thresh %s %s thresh=1" % (eman3dmaskedmap, apix, self.MolMass/factor, eman3dmaskedmap, eman3dmaskedmap)
				if self.WaveInMask: waveproj3dmap=eman3dmaskedmap
				elif self.CCMLInMask: mrcproj3dmap=eman3dmaskedmap
			if self.Verbose: print cmd
			os.system(cmd)

		emanstep=self.MRCRefine[cycle]*2./self.ParticleSize*180./math.pi/3.
		if emanstep>5: emanstep=5 #set at most 5 degree step size
		emanstep=emanstep/float(self.WaveProjFactor)
		self.__WAVEProjStep__=emanstep
		mrcstep=10./self.CCMLProjFactor	# 10degree -> 14 projections, sometimes more are needed

		if cycle==0 or self.SMResolution<=self.MRCRefine[cycle]: cycle2=0
		else: cycle2=cycle
		projimg = "projections-for-cycle-of-%.2fAngstrom-resolution.img" % (self.MRCRefine[cycle2])
		projhed = "projections-for-cycle-of-%.2fAngstrom-resolution.hed" % (self.MRCRefine[cycle2])

		#now creates the Wavelet projections for WaveOrt
		os.chdir(self.__WAVEProjDir__)
		self.__WAVEProjFile__ = os.path.abspath(projimg)
		
		if cycle < self.WaveCycles and (not os.path.exists(projimg) or os.path.getmtime(waveproj3dmap) > os.path.getmtime(projimg)): 
			if os.path.exists(projimg): os.remove(projimg)
			if os.path.exists(projhed): os.remove(projhed)
			#if self.RunPar:
			#	projcmd="projecticosP "
			#else:		
			#	projcmd="projecticos "
			projcmd="projecticos "
			projnum=NumberOfIcosProjections(emanstep)
			#cmd=projcmd+waveproj3dmap+" step="+`emanstep`+" cpu="+`cpu2`+" wave clip="+`self.__CImageSize__`+" scale="+`scale`+Verbose(self.Verbose,thresh=2)
			cmd=projcmd+waveproj3dmap+" out="+projimg+" step="+`emanstep`+" wave clip="+`self.__CImageSize__`+" scale="+`scale`+Verbose(self.Verbose,thresh=2)
			if (cycle==0 and not self.SMCTFCorrect) or (cycle>0 and self.MRCRefine[cycle-1]>self.SMResolution and not self.SMCTFCorrect) or (cycle>0 and self.MRCRefine[cycle-1]<self.SMResolution and not self.CTFCorrect): cmd = cmd + " invert"
			if self.CPU>projnum: 	cpu2=projnum
			else: cpu2=self.CPU
			#if self.Verbose: print cmd
			#os.system(cmd)
			RunPar(cmd,cpus=cpu2,verbose=self.Verbose,workdir=self.ProjectRoot)
			
		#now creates the MRC template projections for refineAll
		os.chdir(self.__MRCProjDir__)
		self.__MRCProjFile__ = os.path.abspath(projimg)
		self.__MRCDataFile__ = os.path.abspath("projections-for-cycle-of-%.2fAngstrom-resolution.dat" % (self.MRCRefine[cycle2]))

		if (not os.path.exists(self.__MRCDataFile__) or not os.path.getsize(self.__MRCDataFile__)) or os.path.getmtime(self.__MRCDataFile__)<os.path.getmtime(mrcproj3dmap): 
			if os.path.exists(projimg): os.remove(projimg)
			if os.path.exists(projhed): os.remove(projhed)
			#if self.RunPar:
			#	projcmd="projecticosP "
			#else:
			#	projcmd="projecticos "
			projcmd="projecticos "
			projnum=NumberOfIcosProjections(mrcstep)
			#cmd=projcmd+mrcproj3dmap+" step="+`mrcstep`+" cpu="+`cpu2`+" random clip="+ \
			cmd=projcmd+mrcproj3dmap+" out="+projimg+" step="+`mrcstep`+" random clip="+`self.__CImageSize__`+" scale="+`scale`+Verbose(self.Verbose,thresh=2)
			if self.CPU>projnum/2: cpu2=projnum/2
			else: cpu2=self.CPU
			RunPar(cmd,cpus=cpu2,verbose=self.Verbose,workdir=self.ProjectRoot)
			
			if os.path.exists(projimg) and ImageInfo(projimg):
				cmd="fftimagic "+projimg+" outort="+self.__MRCDataFile__+" mask="+`self.ParticleSize*self.RefineMaskFactor[-1]/self.__CPixelSize__/2`+" path="+self.__MRCProjDir__+Verbose(self.Verbose,thresh=2)
				if self.Verbose: print cmd
				os.system(cmd)
			else:
				raise ValueError("the projection images \"%s\" are not generated properly" % (os.path.abspath(projimg)))
	def Processed(self, cycle=0):
		"""Check if this \"cycle\" is already processed"""
		mrc3dmap=os.path.join(self.ProjectRoot,self.__MRC3DDir__,"icos5f-refined-to-%.2fAngstrom.mrc" % (self.MRCRefine[cycle]))
		if os.path.exists(mrc3dmap): return 1
		else:	return 0
	def Clean(self):
		"""Clean up the processing directories generated by this script for this project. Used when the processing is restarted from scratch"""
		def safe_delete(project):
			project.PrepareDirectories()
			cmd="rm -rf %s %s %s %s %s" % (project.__MRC3DDir__, project.__MRCProjDir__, project.__MRCDataFile__, project.__WAVEProjDir__, project.__EMANDir__)
			if os.path.exists(os.path.join(project.ProjectRoot,"initmodel-dir")): cmd=cmd+" %s" % (os.path.join(project.ProjectRoot,"initmodel-dir"))
			if project.Verbose: print cmd
			os.system(cmd)
			for m in project.Data:
				cmd="rm -rf %s" % (os.path.join(project.ProjectRoot,m.__id__))
				if m.pair and type(m)==type(m.pair):
					cmd=cmd+" %s" % (os.path.join(project.ProjectRoot,m.pair.__id__))
				if project.Verbose: print cmd
				os.system(cmd)
		if not self.Verbose or (self.Verbose and raw_input("Are you sure to delete all contents in project root dir \""+self.ProjectRoot+"\"(y/n):") =='y'):
			safe_delete(self)
		else:
			print "The project root dir \"%s\" and its contents are not removed !" % self.ProjectRoot
	def IcosReconstruct(self, cycle=0):
		"""	Reconstruct the 3D map from all the individual particles that passed "refineAll" phase residue threshold. Also build the 3D maps
			from the half data sets for resolution assessment with independent reconstruction. Some statistic information are also outputed 
			such as B-factor distribution, refinement yield for each defocus.
		"""
		os.chdir(self.__MRC3DDir__)
		maps_to_build = 0	# to check if all the maps for this cycle have been generated before
		if self.TTest:	# we need to split the data set into two halves
			# now search the closest cycle to the specified resolution
			if cycle==self.__DeepTTestCycle__ and self.DeepTTestLevel>1: tlevel=self.DeepTTestLevel
			else: tlevel=1
			
			for l in range(tlevel):
				denom=int(math.pow(2,l+1))
				if not os.path.exists("icos5f-refined-to-%.2fAngstrom-set-0-of-%d.mrc" % (self.MRCRefine[cycle],denom)): maps_to_build += 1
				if not os.path.exists("icos5f-refined-to-%.2fAngstrom-set-1-of-%d.mrc" % (self.MRCRefine[cycle],denom)): maps_to_build += 1
		if not os.path.exists("icos5f-refined-to-%.2fAngstrom.mrc" % (self.MRCRefine[cycle])): maps_to_build += 1
		if not maps_to_build: return
		
		
		if self.CTFCorrect and self.FixedBFactor:
			self.CreateCTFCurve(cycle=cycle,usefixedB=self.FixedBFactor)
		if os.path.exists("icos5f.mrc"): os.remove("icos5f.mrc")

		diameter=int(int(self.ParticleSize*self.RefineMaskFactor[-1]/self.__CPixelSize__+0.5)/2)*2+1
		if diameter>self.__CImageSize__: diameter=self.__CImageSize__/2*2-1
		if self.__CMapSize__: diameter=self.__CMapSize__
	
		cycle2=cycle
		if cycle!=len(self.MRCRefine)-1: cycle2=cycle+1
		resolution=int(self.__CImageSize__*self.__CPixelSize__/self.MRCRefine[cycle2]+0.5)
		
		AllPtlcsDataFileName="All-particles-refined-to-%.2fAngstrom-ortcen.dat" % (self.MRCRefine[cycle])
		AllPtlcsDataFileP=open(AllPtlcsDataFileName,'w')
		pattern=re.compile(r"^ 0,.*PR.*\]\)=(?P<ccmlp>\s*\d+\.\d*),(?P<scmlp>\s*\d+\.\d*),change:\((?P<dalt>-?\d+\.\d*),(?P<daz>-?\d+\.\d*),(?P<dphi>-?\d+\.\d*),(?P<dx>-?\d+\.\d*),(?P<dy>-?\d+\.\d*)\)$")
		total_particles=0
		for m in self.Data:
			linenum=0
			tnfline=0
			cm=m
			if cycle<self.WaveCycles: 
				p=m.GetFocalPair()
				if p: cm=p
			phase=cm.__ccmlphase__
			angle=self.CCMLOLimits
			dcen=self.CCMLCLimits
			lines=open(cm.__RefinedDataFile__).readlines()
			for l in range(len(lines)):
				if lines[l][0]!='#': 
					if lines[l][:2]==" 0":
						res=pattern.search(lines[l])
						if res:
							ccmlp=string.atof(res.group('ccmlp'))
							dalt=math.fabs(string.atof(res.group('dalt')))
							daz=math.fabs(string.atof(res.group('daz')))
							dphi=math.fmod(math.fabs(string.atof(res.group('dphi'))),360.)
							if dphi>=180: dphi=dphi-180.
							dx=math.fabs(string.atof(res.group('dx')))
							dy=math.fabs(string.atof(res.group('dy')))
							if ccmlp<=phase and (not angle or (angle and dalt<=angle and daz<=angle and dphi<=angle)) and (not dcen or (dcen and dx<=dcen and dy<=dcen)):
								AllPtlcsDataFileP.write(lines[tnfline])
								AllPtlcsDataFileP.write(lines[l])
								linenum=linenum+1
					else: 
						tnfline=l
			cm.__goodimgnum__=linenum
			total_particles+=cm.__goodimgnum__
			if not cm.__goodimgnum__ and self.Verbose:
				print "WARNING: micrograph \"%s\" does not have any good particle found in cycle of (%d of %d) %.2f Angstrom resolution" % (os.path.basename(cm.imagefile), cycle, len(self.MRCRefine), self.MRCRefine[cycle]) 
		AllPtlcsDataFileP.close()
		if not total_particles:
			raise ValueError("no good particle found in cycle of (%d of %d) %.2f Angstrom resolution" % (cycle, len(self.MRCRefine), self.MRCRefine[cycle])) 

		self.PlotOrientationDistribution(filename=AllPtlcsDataFileName)
		self.PlotDefocusYieldCurve(filename="All-particles-refined-to-%.2fAngstrom-yields.agr" % (self.MRCRefine[cycle]))
		self.PlotCTFIntensityAvgCurve(filename="All-particles-refined-to-%.2fAngstrom-avg-ctf.agr" % (self.MRCRefine[cycle]))
		
		memo="\""+self.Sample +" refined to %.2fAngstrom and reconstructed to %.2fAngstrom resolution" % (self.MRCRefine[cycle],self.MRCRefine[cycle2])+"\""
		
		FileName=[]
		MapName=[]
		if self.TTest:	# we need to split the data set into two halves
			# now search the closest cycle to the specified resolution
			if cycle==self.__DeepTTestCycle__ and self.DeepTTestLevel>1: tlevel=self.DeepTTestLevel
			else: tlevel=1
			
			FileP=[]
			for l in range(tlevel):
				denom=int(math.pow(2,l+1))
				FileName.append("All-particles-refined-to-%.2fAngstrom-ortcen-set-0-of-%d.dat" % (self.MRCRefine[cycle],denom))
				FileP.append(open(FileName[-1],'w'))
				FileName.append("All-particles-refined-to-%.2fAngstrom-ortcen-set-1-of-%d.dat" % (self.MRCRefine[cycle],denom))
				FileP.append(open(FileName[-1],'w'))
				MapName.append("icos5f-refined-to-%.2fAngstrom-set-0-of-%d.mrc" % (self.MRCRefine[cycle],denom))
				MapName.append("icos5f-refined-to-%.2fAngstrom-set-1-of-%d.mrc" % (self.MRCRefine[cycle],denom))
			FileP0=open(AllPtlcsDataFileName,'r')
			lines=FileP0.readlines()
			FileP0.close()
			for lineid in range(len(lines)/2):
				for l in range(tlevel):
					denom=int(math.pow(2,l+1))
					mod=lineid%denom
					if mod<2: 
						FileP[l*2+mod].write(lines[2*lineid])
						FileP[l*2+mod].write(lines[2*lineid+1])
			for fp in FileP: fp.close()
			
		# now the reconstruction with all particles	
		FileName.append(AllPtlcsDataFileName)
		MapName.append("icos5f-refined-to-%.2fAngstrom.mrc" % (self.MRCRefine[cycle]))
		if self.CPU>resolution/10: cpu1=resolution/10	# temporary solution
		else: cpu1=self.CPU
		if self.CPU>self.MapSize/10: cpu2=self.MapSize/10
		else: cpu2=self.CPU
		if cpu2<2: cpu2=2	# very stupid, but I have to do this since the emicofb step of reconstruction code has to use at least 2 cpus

		scale=1
		if cycle!=len(self.MRCRefine)-1:
			cshrink=self.ShrinkFactor(cycle)
			nshrink=self.ShrinkFactor(cycle+1)
			if nshrink<cshrink: scale=float(cshrink)/float(nshrink)
				
		for set in range(len(FileName)):
			cmap=MapName[set]
			if os.path.exists(cmap):
				if self.Verbose: print cmap+" exists, skipped\n"
			else:
				#cmd="reconstruct.py "+FileName[set]+" --script=all-particle-reconstruct.script --diameter="+`diameter`+" --res="+`resolution`+" --cpu="+`cpus`+" --memo="+memo
				cmd="reconstruct.py %s --script=%s --diameter=%d --res=%f --cpu1=%d --cpu2=%d --memo=%s" % \
					 (FileName[set],FileName[set][:string.find(FileName[set],"ortcen")]+"reconstruction.com",diameter,resolution,cpu1,cpu2,memo)
				if self.CTFCorrect: cmd=cmd+" --ctf="+self.__MRCCTFFile__
				if cycle==len(self.MRCRefine)-1 and set==len(FileName)-1:
					cmd=cmd+" --maps=3"
					scale=2
				if scale!=1:    cmd = cmd + " --scale=%f" % scale
				if self.Verbose: print cmd
				os.system(cmd)
				#if os.path.exists(cmap):	os.remove(cmap)
				if os.path.exists("icos3f.mrc"): os.rename("icos3f.mrc",cmap.replace("icos5f","icos3f"))
				if os.path.exists("icos2f.mrc"): os.rename("icos2f.mrc",cmap.replace("icos5f","icos2f"))
				if os.path.exists("icos5f.mrc"): os.rename("icos5f.mrc",cmap)
				else: raise RuntimeError("during running command \"%s\"" % (cmd))
		if self.Mail: 
			from socket import gethostname, gethostbyaddr
			msg="The 3D reconstruction map is on computer \"%s\": \"%s\"" % (gethostbyaddr(gethostname())[0],os.path.abspath(MapName[-1]))
			msg=msg+"\n\n"+(self.RefinementStats(FileName[-1]))["msg"]
			if cycle == len(self.MRCRefine)-1: msg="Congratulations! The processing is now completed!\n\n"+msg
			self.SendMail(cycle=cycle, msg=msg)
	def SendMail(self, cycle, msg=""):
		if not len(self.MRCRefine): self.SetupRefineCycles()
		import time, tempfile
		images=[]
		if self.TTest:
			fscret=self.PlotFourierShellCorrelation(plotnow=0)
			xmgrfile=fscret["xmgrfile"]
			#cycle=fscret["lastcycle"]
			if os.path.exists(xmgrfile): 
				image=tempfile.mktemp(".jpg")
				try:
					cmd="gracebat -hardcopy -printfile %s -hdevice JPEG %s" % (image, xmgrfile)
					if self.Verbose: print cmd
					os.system(cmd)
					if os.path.exists(image): images.append(image)
				except:
					pass
		mapfile=os.path.abspath("icos5f-refined-to-%.2fAngstrom.mrc" % (self.MRCRefine[cycle]))
		if os.path.exists(mapfile): 
			image=tempfile.mktemp(".png")
			try:
				cmd="map2img %s %s" % (mapfile, image)
				if self.Verbose: print cmd
				os.system(cmd)
				if os.path.exists(image): images.append(image)
			except:
				pass
		subject="%s: finished refinement cycle of %.2f Angstrom (%d of %d cycles) at %s" % (self.Sample, self.MRCRefine[cycle], cycle+1, len(self.MRCRefine),time.strftime("%c",time.localtime(time.time())))
		if images: msg=msg+"\n\nThe Fourier Shell Correlation curve and the central section of \"%s\" are:" % (mapfile)
		SendMail(to=self.Mail,subject=subject,msg=msg, images=images)
		for image in images: 
			if os.path.exists(image): os.remove(image)                 
	def Run(self,reset=0):
		"""Run the whole processing steps to get 3D reconstructions"""
		import time, socket
		#if self.IgnoreSrcImgs: self.IgnoreSourceImages()
		self.SetupRefineCycles()
		self.PrepareDirectories()
		if reset>=self.MRCRefine[-1] and reset<self.MRCRefine[0]:
			self.Reset2CycleOfResolution(res=reset)
		try:
			timefile=open(os.path.join(self.ProjectRoot,"time.log"),"a")
			timefile.write("\n%s a new run begins %s\n" % ('#'*30,'#'*30)) 
		except:
			timefile=open(os.path.join(self.ProjectRoot,"time.log"),"w")
		pstime=time.time()
		timefile.write("\nSAVR: automated icosahedral reconstruction of \"%s\" on \"%s\" with %d CPUs\n" % (self.Sample, socket.gethostname(), self.CPU))
		timefile.write("Resolution range: %.2f Angstrom -> %.2f Angstrom\n" % (self.MRCRefine[0],self.MRCRefine[-1]))
		timefile.write("Refinement cycles (%d): %s\n\n" % (len(self.MRCRefine), `self.MRCRefine`))
		timefile.write("Start time: %s\n" % (time.ctime(pstime)))
		timefile.flush()
		
		try:
			for cycle in range(len(self.MRCRefine)):
				if self.Verbose: print "Starting cycle %d" % cycle
				cstime=time.time()
				timefile.write("\nCycle %2d (%.2f Angstrom) starts at: %s" % (cycle, self.MRCRefine[cycle], time.ctime(cstime)))
				if self.MultSampling: 
					timefile.write(", with images shrunk by %d times\n" % (self.ShrinkFactor(cycle)))
				else: 
					timefile.write("\n")
				timefile.flush()
				self.SetupParameters()
				self.PrepareProjectionImages(cycle=cycle)
				self.PrepareParticleImages(cycle=cycle)
				if cycle==0: self.PlotDefocusBFactorCurve(os.path.join(self.ProjectRoot,"DefocusBFactorCurve.agr"), plotnow=0)
				if self.RunPar:
					cmd=""
					for m in self.Data:	# loop through all micrographs
						if cycle==0 and not self.WaveCycles:
							tcmd=m.SCMLEstimateOrientation(self)
							if tcmd: cmd=cmd+tcmd+";"
						elif cycle<self.WaveCycles:
							tcmd=m.WaveProjectionMatching(self,cycle=cycle)
							if tcmd: cmd=cmd+tcmd+";"
						tcmd=m.CCMLRefine(self,cycle=cycle)
						if tcmd: cmd=cmd+tcmd+"\n"
					if cmd:
						stime=time.time()
						timefile.write("Cycle %2d orientation search starts at: %s, " % (cycle, time.ctime(stime))) 
						timefile.flush()
						self.RunParCommands(cmd, jobfile=os.path.join(self.ProjectRoot,"runpar-cycle-%d-of-%d.runpar" % (cycle, len(self.MRCRefine))))
						etime=time.time()
						timefile.write("finishes at: %s, total %d seconds (%.2f hours)\n" % (time.ctime(etime), etime-stime, float(etime-stime)/3600.)) 
						timefile.flush()
				else:
					stime=time.time()
					timefile.write("Cycle %2d orientation search starts at: %s, " % (cycle, time.ctime(stime))) 
					timefile.flush()
					for m in self.Data:	# loop through all micrographs
						if cycle==0 and not self.WaveCycles:
							m.SCMLEstimateOrientation(self)
						elif cycle<self.WaveCycles:
							m.WaveProjectionMatching(self,cycle=cycle)
						m.CCMLRefine(self,cycle=cycle)
						etime=time.time()
						timefile.write("finishes at: %s, total %d seconds (%.2f hours)\n" % (time.ctime(etime), etime-stime, float(etime-stime)/3600.)) 
						timefile.flush()
				mrc3dmap=os.path.join(self.ProjectRoot,self.__MRC3DDir__,"icos5f-refined-to-%.2fAngstrom.mrc" % (self.MRCRefine[cycle]))
				if (cycle>=self.WaveCycles-1) and ((self.SMResolution>0 and (self.MRCRefine[cycle]<self.SMResolution or (cycle<len(self.MRCRefine)-1 and self.MRCRefine[cycle+1]<self.SMResolution ))) or self.SMResolution<=0):
					stime=time.time()
					timefile.write("Cycle %2d 3D map reconstruction starts at: %s, " % (cycle, time.ctime(stime))) 
					timefile.flush()
					self.IcosReconstruct(cycle=cycle)
					etime=time.time()
					timefile.write("finishes at: %s, total %d seconds (%.2f hours)\n" % (time.ctime(etime), etime-stime, float(etime-stime)/3600.))
					cetime=time.time()
					timefile.write("Cycle %2d finishes at: %s, total %d seconds (%.2f hours)\n" % (cycle, time.ctime(cetime), cetime-cstime, float(cetime-cstime)/3600.))
					timefile.flush()
		except SystemExit, KeyboardInterrupt:
			timefile.write("\nThe execution aborted at: %s\n" % (time.ctime(time.time())))
			timefile.close()
		else:
			petime=time.time()
			timefile.write("\nThe whole project is done at: %s, total %d seconds (%.2f hours)\n" % (time.ctime(petime), petime-pstime, float(petime-pstime)/3600.))
			timefile.close()                    

	def Reset2CycleOfResolution(self,res=0):
		"""	Rollback the processing to certain resolution step. Useful if something has gone wrong and need to go back some steps.
			The \"res\" should be the resolution that is incorrectly processed and must be dumped, it doesn't need to be exact as long as 
			it is close enough in value to the targeted resolution
		"""
		if res<=0: 
			print "In Project.Reset2CycleOfResolution(self,res=0): incorrect 'res="+res+"', 'res' should large than 0. ignored\n"
			return
		
		# check if we have all the intermediate variables set		
		dir_set=1
		for m in self.Data:
			if not (m.__emanworkdir__ and m.__mrcworkdir__) or \
			   (m.pair and not (m.pair.__emanworkdir__ and m.pair.__mrcworkdir__)):
				dir_set=0
				break
		if not dir_set: self.PrepareDirectories()
		
		self.SetupRefineCycles()
		if res>self.MRCRefine[0] or res<self.MRCRefine[-1]:
			print "In Project.Reset2CycleOfResolution(self,res=0): incorrect 'res="+res+"', it is out of range [%.2f,%.2f]. ignored\n" % (self.MRCRefine[0],self.MRCRefine[-1])			
			return
		# now search the closest cycle to the specified resolution
		refine2=copy.copy(self.MRCRefine)
		for i in range(len(refine2)): refine2[i]=math.fabs(refine2[i]-res)
		startcycle=refine2.index(min(refine2))
		
		# now delete all the results from cycles of higher resolutions
		for cycle in range(startcycle,len(self.MRCRefine)):
			datafiles="*-%.2fAngstrom*.*" % (self.MRCRefine[cycle])
			cmd="\\find %s -name \"%s\" -exec rm {} \\;" % (self.ProjectRoot,datafiles)
			#cmd="\\find %s -name \"%s\" -exec ls -l {} \\;" % (self.ProjectRoot,datafiles)
			print cmd
			os.system(cmd)
		
		# now reset the orientation and center parameters in the image headers
		for m in self.Data:			
			if startcycle>=self.WaveCycles:
				imgbase=os.path.basename(os.path.splitext(m.imagefile)[0])
				nimgfile=os.path.join(m.__emanworkdir__,imgbase+".img")
				out=os.path.join(m.__mrcworkdir__,imgbase+"-refined-to-res-%.2fAngstrom.dat" % (self.MRCRefine[startcycle-1]))
				if os.path.exists(out):
					cmd="OrtCen2Imagic "+out+" "+nimgfile+Verbose(self.Verbose,thresh=2)
					if self.Verbose: print cmd
					os.system(cmd)
		return startcycle
	def PlotFourierShellCorrelation(self, filename="FourierShellCorrelation.agr", plotnow=1):
		"""Compute the Fourier Shell Correlation curve for all the independent reconstruction 3D map pairs and return the curves in a xmgrace file"""
		try:	
			self.SetupRefineCycles()
			os.chdir(os.path.join(self.ProjectRoot,"mrc-3d-dir"))
			header="# Grace project file\n"
			header=header+ "@    title \"%s: Fourier Shell Correlation\"\n" % (self.Sample)
			header=header+ "@    xaxis  label \"Resolution(\cE\C)\"\n"
			resolutions=[40,30,20,15,12,10,9,8,7,6,5,4]
			header=header+ "@    xaxis  tick spec type both\n"
			header=header+ "@    xaxis  tick spec %d\n" % len(resolutions)
			for i in range(len(resolutions)):
				header=header+ "@    xaxis  ticklabel %d, \"%d\"\n" % (i, float(resolutions[i]))
				header=header+ "@    xaxis  tick major %d,\t%f\n" % (i, 1./float(resolutions[i]))
			header=header+ "@    yaxis  label \"Fourier Shell Correlation\"\n"
			header=header+ "@    world xmin 0\n"
			#header=header+ "@    world xmax %f\n" % ( float(int(1./self.MRCRefine(-1)*100+0.5))/100.0 )
			header=header+ "@    world ymin 0\n"
			header=header+ "@    world ymax 1\n" 
			header=header+ '@map color 1 to (0, 0, 0), "black"\n'
			header=header+ '@map color 2 to (255, 0, 0), "red"\n'
			header=header+ '@map color 3 to (0, 255, 0), "green"\n'
			header=header+ '@map color 4 to (0, 0, 255), "blue"\n'
			header=header+ '@map color 5 to (0, 255, 255), "cyan"\n'
			header=header+ '@map color 6 to (148, 0, 211), "violet"\n'
			header=header+ '@map color 7 to (255, 0, 255), "magenta"\n'
			header=header+ '@map color 8 to (255, 165, 0), "orange"\n'
			header=header+ '@map color 9 to (114, 33, 188), "indigo"\n'
			header=header+ '@map color 10 to (103, 7, 72), "maroon"\n'
			header=header+ '@map color 11 to (64, 224, 208), "turquoise"\n'
			header=header+ '@map color 12 to (0, 139, 0), "green4"\n'
			xmgrfile=open(filename,'w')
			xmgrfile.write(header)

			cpus_max = int( 64*math.pow(self.ImageSize,3)/1000000000L)
			cpus=self.CPU
			if cpus>cpus_max: cpus=cpus_max	# too many number of CPUs actually hurt performence 
			
			dataset=0
			lastcycle=0
			for cycle in range(len(self.MRCRefine)):
				if self.MultSampling and cycle!=len(self.MRCRefine)-1:
					shrink=self.ShrinkFactor(cycle+1)
				else: 
					shrink=1
				self.__CPixelSize__=self.PixelSize*shrink
				self.__CImageSize__=int(self.ImageSize/shrink)/4*4

				if self.DeepTTestLevel>1: tlevel=self.DeepTTestLevel
				else: tlevel=1
				for l in range(tlevel):
					denom=int(math.pow(2,l+1))
					set0file="icos5f-refined-to-%.2fAngstrom-set-0-of-%d.mrc" % (self.MRCRefine[cycle], denom)
					set1file="icos5f-refined-to-%.2fAngstrom-set-1-of-%d.mrc" % (self.MRCRefine[cycle], denom)
	
					if l==0:
						fscfile ="icos5f-refined-to-%.2fAngstrom-fsc.txt" % (self.MRCRefine[cycle])
					else:
						fscfile ="icos5f-refined-to-%.2fAngstrom-fsc-of-%d.txt" % (self.MRCRefine[cycle],denom)
					
					if (os.path.exists(set0file) and os.path.exists(set1file)) or os.path.exists(fscfile): 
						xmgrfile.write("@target G0.S%d\n" % (dataset))
						if l==0:
							xmgrfile.write("@    s%d legend  \"refined to %.2f\cE\C\"\n" % (dataset,self.MRCRefine[cycle]))
						else:
							xmgrfile.write("@    s%d legend  \"refined to %.2f\cE\C (%d sets)\"\n" % (dataset,self.MRCRefine[cycle],denom))
						xmgrfile.write("@type xy\n")
						if os.path.exists(fscfile): pass
						elif (os.path.exists(set0file) and os.path.exists(set1file)):
							if self.Symmetrize:
								cmd="symmetrizeP.py %s tmp0.mrc --mrc --sym=icos --cpu=%d " % (set0file, self.CPU)
								#cmd=cmd+"proc3d %s.sym.img tmp0.mrc ;" % (set0file)
								#cmd=cmd+"hlf5fold2full5fold %s.sym.img tmp0-full.mrc resize=%d ;" % (set0file,self.__CImageSize__)
								#cmd=cmd+"rm %s.sym.img %s.sym.hed ;" % (set0file, set0file)
							else:
								cmd="cp %s tmp0.mrc" % (set0file)
								#cmd=cmd+"hlf5fold2full5fold %s tmp0-full.mrc resize=%d ;" % (set0file,self.__CImageSize__)
							if self.FSCInMask and self.InnerRadius>0:
								cmd=cmd+" ; icosmask tmp0.mrc tmp0.mrc apix="+`self.__CPixelSize__`+" inmask="+`self.InnerRadius`+ \
									","+`self.ICFactor*self.InnerRadius`+" outmask="+`self.ParticleSize*self.RefineMaskFactor[-1]/2.0`+ \
									","+`self.ICFactor*self.ParticleSize/2`+" soft="+`self.IMaskSlope`
							if self.MolMass: cmd=cmd+" ; volume tmp0.mrc "+`self.__CPixelSize__`+" set=%f ; thresh tmp0.mrc tmp0.mrc thresh=1" % (self.MolMass/2.0)
							if self.Symmetrize:
								cmd=cmd+" ; symmetrizeP.py %s tmp1.mrc --mrc --sym=icos --cpu=%d" % (set1file, self.CPU)
								#cmd=cmd+"proc3d %s.sym.img tmp1.mrc ;" % (set1file)
								#cmd=cmd+"hlf5fold2full5fold %s.sym.img tmp1-full.mrc resize=%d ;" % (set1file,self.__CImageSize__)
								#cmd=cmd+"rm %s.sym.img %s.sym.hed ;" % (set1file, set1file)
							else:
								cmd=cmd+"; cp %s tmp1.mrc" % (set1file)
								#cmd=cmd+"hlf5fold2full5fold %s tmp1-full.mrc resize=%d ;" % (set1file,self.__CImageSize__)
							#cmd=cmd+"hlf5fold2full5fold "+set1file+" tmp1-full.mrc resize="+`self.__CImageSize__`+";"
							if self.FSCInMask and self.InnerRadius>0:
								cmd=cmd+" ; icosmask tmp1.mrc tmp1.mrc apix="+`self.__CPixelSize__`+" inmask="+`self.InnerRadius`+ \
									","+`self.ICFactor*self.InnerRadius`+" outmask="+`self.ParticleSize*self.RefineMaskFactor[-1]/2.0`+ \
									","+`self.ICFactor*self.ParticleSize/2`+" soft="+`self.IMaskSlope`
							if self.MolMass: cmd=cmd+" ; volume tmp1.mrc "+`self.__CPixelSize__`+" set=%f ; thresh tmp1.mrc tmp1.mrc thresh=1" % (self.MolMass/2.0)
							#cmd=cmd+"proc3d tmp0-full.mrc tmp1-full.mrc fsc="+fscfile
							cmd=cmd+" ; fscP.py tmp0.mrc tmp1.mrc %s --resize=%d --cpu=%d" % (fscfile,self.__CImageSize__,cpus)
							#cmd=cmd+" ; rm -f tmp0-full.mrc tmp1-full.mrc"
							cmdlog=open("FourierShellCorrelation-Cycle-of-%.2fAngstrom-set-of-%d.com" % (self.MRCRefine[cycle],denom),"w")
							cmdlog.write(cmd)
							cmdlog.close()
							if self.Verbose: print cmd
							os.system(cmd)
						fscp=open(fscfile,'r')
						curve=fscp.readlines()
						fscp.close()
						invstep=1./(self.__CImageSize__*self.__CPixelSize__)
						for row in curve:
							x,y=string.split(row)
							x=float(x)*invstep
							if x>0 and float(y)>0: xmgrfile.write("\t%f\t%s\n" % (x,y))
						xmgrfile.write("&\n")
						dataset=dataset+1
						lastcycle=cycle
			if dataset>5: # only display 5 curves to avoid a crowded graph, the user could select to display the other hidden curves in xmgrace
				for set in range(dataset-5): xmgrfile.write("@\ts%d hidden true\n" % ( set ))
			xmgrfile.close()
			if self.Verbose:
				print "Fourier Shell Correlation curves are saved in xmgrace plot \""+os.path.abspath(filename)+\
					  "\", please run command \"xmgrace -free "+os.path.abspath(filename)+"\" to view the curves"
			if plotnow:
				os.system("xmgrace -free %s" % (filename))
			return {"xmgrfile":os.path.abspath(filename),"lastcycle":lastcycle}
		except KeyboardInterrupt:
			return None
	def PlotOrientationDistribution(self, filename, plotnow=0):
		"""Find the orientations for all the individual particles and output in a xmgrace file"""
		if len(filename) and os.path.exists(filename):
			try:
				points=[]
				fp=open(filename,'r')
				lines=fp.readlines()
				fp.close()
				for line in lines:
					line=string.strip(line)
					fields=string.split(line,',')
					if len(fields)<3 or fields[0][0]=='#': pass
					else:
						points.append([fields[1],fields[2]])
	
				from stat import ST_UID
				if os.getuid() != os.stat(filename)[ST_UID] or os.getuid() != os.stat(os.curdir)[ST_UID]:
					xmgrfile=os.path.join("/tmp",os.path.basename(os.path.splitext(filename)[0])+"-dist.agr")
				else:
					xmgrfile=os.path.splitext(filename)[0]+"-dist.agr"
				if self.Verbose:
					print "There are %d particles in \"%s\"" % (len(points),filename) 
					print "The xmgrace file is saved as \"%s\"" % (xmgrfile)
				xfp=open(xmgrfile,'w')
				header="# Grace project file\n#\n"
				header=header+ "@with g0\n"
				header=header+ "@    title \"%s: Particle Orientation Distribution\"\n" % self.Sample
				header=header+ "@    subtitle \"%d particles\"\n" % len(points)
				header=header+ "@    xaxis  label \"\\xa\"\n"
				header=header+ "@    yaxis  label \"\\xq\"\n"
				header=header+ "@    world xmin -32\n"
				header=header+ "@    world xmax 32\n"
				header=header+ "@    world ymin 90\n"
				header=header+ "@    world ymax 69\n"
				header=header+ "@    yaxes invert on\n"
				header=header+ "@    xaxis tick on\n"
				header=header+ "@    xaxis tick major 10\n"
				header=header+ "@    xaxis tick minor ticks 1\n"
				header=header+ "@    yaxis tick on\n"
				header=header+ "@    yaxis tick major 5\n"
				header=header+ "@    yaxis tick minor ticks 1\n"
				header=header+ "@    s0 type xy\n"
				header=header+ "@    s0 symbol 1\n"
				header=header+ "@    s0 symbol size 0.3\n"
				header=header+ "@    s0 line type 0\n"
				header=header+ "@type xy\n"
				for i in points:
					header=header+"\t%s\t%s\n" % (i[1],i[0])
				header=header+"&\n"
				xfp.write(header)
				xfp.close()
				if self.Verbose:
					print "The orientations of all individual partciles are saved in xmgrace plot \""+os.path.abspath(filename)+\
					 "\", please run command \"xmgrace -free "+os.path.abspath(filename)+"\" to view the distribution"
				if plotnow: os.system("xmgrace -free %s" % (xmgrfile))
				return points
			except KeyboardInterrupt:
				return None
		else:
			return None
	def PlotDefocusYieldCurve(self, filename, plotnow=0):
		"""Find the orientations for all the individual particles and output in a xmgrace file"""
		try:
			percentage=[]
			tgoodimgnum=0
			timagenum=0
			for m in self.Data:
				tgoodimgnum=tgoodimgnum+m.__goodimgnum__
				timagenum=timagenum+m.imagenum
				percentage.append([m.defocus, float(m.__goodimgnum__)/float(m.imagenum)])
			#percentage.sort(lambda m1,m2: -1*(math.fabs(m1[0])<math.fabs(m2[0])))
			percentage.sort(lambda m1,m2: cmp(math.fabs(m1[0]),math.fabs(m2[0])))
				
			header="# Grace project file\n#\n"
			header=header+ "@with g0\n"
			header=header+ "@    title \"%s: Refinement Yield\"\n" % self.Sample
			header=header+ "@    xaxis  label \"Defocus (\\c5\\Cm)\"\n"
			header=header+ "@    subtitle \"%d micrographs, %d particles, %d good, average yield %.2f\"\n" % (len(percentage),timagenum,tgoodimgnum,float(tgoodimgnum)/float(timagenum))
			header=header+ "@    yaxis  label \"Fraction\"\n"
			header=header+ "@    world ymin 0\n"
			header=header+ "@    world ymax 1\n"
			header=header+ "@    s0 type xy\n"
			header=header+ "@    s0 symbol 2\n"
			header=header+ "@    s0 symbol size 1\n"
			header=header+ "@    s0 line type 0\n"
			header=header+ "@type xy\n"
				
			fp=open(filename,"w")
			fp.write(header)
			for p in percentage:
				fp.write("\t%f\t%f\n" % (p[0], p[1]))
			fp.write("&\n")
			fp.close()	
			if self.Verbose:
				print "The refinement yield vs defocus curve is written to file \"%s\", please run command \"xmgrace -free %s\" to view it" % (os.path.abspath(filename),os.path.abspath(filename))
			if plotnow:
				os.system("xmgrace -free %s" % (filename))	
			return percentage
		except KeyboardInterrupt:
			return None
	def PlotCTFIntensityAvgCurve(self, filename, plotnow=0):
		"""Compute the average CTF intensity curve from all the individual particles and output in a xmgrace file"""
		try:
			ctfsum=[]
			totimgnum=0
			for i in range(len(self.Data)):
				m=self.Data[i]
				ctfcurve=CTFCurve(defocus=-1*m.defocus,B=m.B,ampcontrast=self.AmpContrast,voltage=self.Voltage,Cs=self.Cs,apix=self.PixelSize,imagesize=self.ImageSize,res=self.PixelSize*2.0,mode=1)	
				if m.__goodimgnum__: imgnum=m.__goodimgnum__
				else: imgnum=m.ImageInfo(usesrcimg=1)["num"]
				
				totimgnum=totimgnum+imgnum
				
				for pixel in ctfcurve: pixel[1]=pixel[1]*imgnum
				
				if i==0: ctfsum=ctfcurve
				else: 
					for p in range(len(ctfcurve)): 
						ctfsum[p][1]=ctfsum[p][1]+ctfcurve[p][1]
			for p in range(len(ctfsum)): 
				ctfsum[p][1]=ctfsum[p][1]/float(totimgnum)
				
			header="# Grace project file\n#\n"
			header=header+ "@with g0\n"
			header=header+ "@    title \"%s: Average CTF Intensity\"\n" % self.Sample
			header=header+ "@    subtitle \"%d micrographs, %d particles\"\n" % (len(self.Data), totimgnum)
			resolutions=[30,20,15,12,10,9,8,7,6,5]
			header=header+ "@    xaxis  tick spec type both\n"
			header=header+ "@    xaxis  tick spec %d\n" % len(resolutions)
			for i in range(len(resolutions)):
				header=header+ "@    xaxis  ticklabel %d, \"%d\"\n" % (i, float(resolutions[i]))
				header=header+ "@    xaxis  tick major %d,\t%f\n" % (i, 1./float(resolutions[i]))
			header=header+ "@    xaxis  label \"s(1/\cE\C)\"\n"
			header=header+ "@    yaxis  label \"Intensity\"\n"
			header=header+ "@    world ymin 0\n"
			header=header+ "@    world ymax 1\n"
			header=header+ "@    s0 type xy\n"
			header=header+ "@type xy\n"
				
			fp=open(filename,"w")
			fp.write(header)
			for p in ctfsum:
				fp.write("\t%f\t%f\n" % (p[0], p[1]))
			fp.write("&\n")
			fp.close()	
			if self.Verbose:
				print "The average CTF intensity curve is written to file \"%s\", please run command \"xmgrace -free %s\" to view it" % (os.path.abspath(filename),os.path.abspath(filename))
			if plotnow:
				os.system("xmgrace -free %s" % (filename))	
			return ctfsum
		except KeyboardInterrupt:
			return None
	def PlotDefocusBFactorCurve(self, filename, plotnow=1):
		"""Sort the B factor according to the defocus for all the micrographs and output in a xmgrace file"""
		try:
			bcurve=[]
			for m in self.Data:
				bcurve.append([m.defocus, m.B])
			#bcurve.sort(lambda m1,m2: -1*(math.fabs(m1[0])<math.fabs(m2[0])))
			bcurve.sort(lambda m1,m2: cmp(math.fabs(m1[0]),math.fabs(m2[0])))
				
			header="# Grace project file\n#\n"
			header=header+ "@with g0\n"
			header=header+ "@    title \"%s: Experimental B Factor Distribution\"\n" % self.Sample
			header=header+ "@    subtitle \"%d micrographs\"\n" % len(self.Data)
			header=header+ "@    xaxis  label \"Defocus (\\c5\\Cm)\"\n"
			header=header+ "@    yaxis  label \"Experimental B Factor\"\n"
			header=header+ "@    s0 type xy\n"
			header=header+ "@    s0 symbol 2\n"
			header=header+ "@    s0 symbol size 1\n"
			header=header+ "@    s0 line type 0\n"
			header=header+ "@type xy\n"
				
			fp=open(filename,"w")
			fp.write(header)
			for p in bcurve:
				fp.write("\t%f\t%f\n" % (p[0], p[1]))
			fp.write("&\n")
			fp.close()	
			if self.Verbose:
				print "The B factor vs defocus curve is written to file \"%s\", please run command \"xmgrace -free %s\" to view it" % (os.path.abspath(filename),os.path.abspath(filename))
			if plotnow: os.system("xmgrace -free %s" % (filename))	
			return bcurve
		except KeyboardInterrupt:
			return None
	def RefinementStats(self, filename):
		"""Compute some statistic information from the refinement result such as average phase residues and angle changes"""
		if len(filename) and os.path.exists(filename):
			try:
				fp=open(filename,'r')
				lines=fp.readlines()
				fp.close()
				
				max_ccmlp=0.
				max_scmlp=0.
				max_dalt=0.
				max_daz=0.
				max_dphi=0.
				max_dx=0.
				max_dy=0.

				min_ccmlp=180.
				min_scmlp=180.
				min_dalt=720.
				min_daz=720.
				min_dphi=720.
				min_dx=1000.
				min_dy=1000.

				from array import array
				ccmlp=array('d')
				scmlp=array('d')
				dalt=array('d')
				daz=array('d')
				dphi=array('d')
				dx=array('d')
				dy=array('d')

				sumccmlp=0;	sum2ccmlp=0
				sumscmlp=0;	sum2scmlp=0
				sumdalt=0;	sum2dalt=0
				sumdaz=0;	sum2daz=0
				sumdphi=0;	sum2dphi=0
				sumdx=0;	sum2dx=0
				sumdy=0;	sum2dy=0

				pattern=re.compile(r"^ 0,.*PR.*\]\)=(?P<ccmlp>\s*\d+\.\d*),(?P<scmlp>\s*\d+\.\d*),change:\((?P<dalt>-?\d+\.\d*),(?P<daz>-?\d+\.\d*),(?P<dphi>-?\d+\.\d*),(?P<dx>-?\d+\.\d*),(?P<dy>-?\d+\.\d*)\)$")
				for line in lines:
					res=pattern.search(line)
					if res: 
						ccmlp.append(string.atof(res.group('ccmlp')))
						sumccmlp=sumccmlp+ccmlp[-1]
						sum2ccmlp=sum2ccmlp+ccmlp[-1]*ccmlp[-1]
						if ccmlp[-1] < min_ccmlp: min_ccmlp=ccmlp[-1]
						if ccmlp[-1] > max_ccmlp: max_ccmlp=ccmlp[-1]
						
						scmlp.append(string.atof(res.group('scmlp')))
						sumscmlp=sumscmlp+scmlp[-1]
						sum2scmlp=sum2scmlp+scmlp[-1]*scmlp[-1]
						if scmlp[-1] < min_scmlp: min_scmlp=scmlp[-1]
						if scmlp[-1] > max_scmlp: max_scmlp=scmlp[-1]
						
						dalt.append(math.fabs(string.atof(res.group('dalt'))))
						sumdalt=sumdalt+dalt[-1]
						sum2dalt=sum2dalt+dalt[-1]*dalt[-1]
						if dalt[-1] < min_dalt: min_dalt=dalt[-1]
						if dalt[-1] > max_dalt: max_dalt=dalt[-1]
						
						daz.append(math.fabs(string.atof(res.group('daz'))))
						sumdaz=sumdaz+daz[-1]
						sum2daz=sum2daz+daz[-1]*daz[-1]
						if daz[-1] < min_daz: min_daz=daz[-1]
						if daz[-1] > max_daz: max_daz=daz[-1]
						
						dphi.append(math.fmod(math.fabs(string.atof(res.group('dphi'))),360.))
						if dphi[-1]>=180: dphi[-1]=dphi[-1]-180.
						sumdphi=sumdphi+dphi[-1]
						sum2dphi=sum2dphi+dphi[-1]*dphi[-1]
						if dphi[-1] < min_dphi: min_dphi=dphi[-1]
						if dphi[-1] > max_dphi: max_dphi=dphi[-1]
						
						dx.append(math.fabs(string.atof(res.group('dx'))))
						sumdx=sumdx+dx[-1]
						sum2dx=sum2dx+dx[-1]*dx[-1]
						if dx[-1] < min_dx: min_dx=dx[-1]
						if dx[-1] > max_dx: max_dx=dx[-1]
						
						dy.append(math.fabs(string.atof(res.group('dy'))))
						sumdy=sumdy+dy[-1]
						sum2dy=sum2dy+dy[-1]*dy[-1]	
						if dy[-1] < min_dy: min_dy=dy[-1]
						if dy[-1] > max_dy: max_dy=dy[-1]
				if self.Data:
					timagenum=0
					for m in self.Data:
						timagenum=timagenum+m.ImageInfo()["num"]
					msg="There are %d particles in \"%s\" out of total %d particles (%.2f%%) in all micrographs\n" % (len(ccmlp),filename,timagenum,100.*len(ccmlp)/timagenum) 
				else:
					msg="There are %d particles in \"%s\"\n" % (len(ccmlp),filename) 

				mean=sumccmlp/len(ccmlp)
				sigma=math.sqrt(sum2ccmlp/len(ccmlp)-mean*mean)
				ret={"ccmlpr": [mean, sigma, max_ccmlp, min_ccmlp]}
				msg=msg+"CCML Phase Residue: Mean = %7.3f\tSigma = %7.3f\tMax = %7.3f\tMin = %7.3f\n" % (mean,sigma,max_ccmlp,min_ccmlp) 
				
				mean=sumscmlp/len(scmlp)
				sigma=math.sqrt(sum2scmlp/len(scmlp)-mean*mean)
				ret["scmlpr"]=[mean, sigma, max_scmlp, min_scmlp]
				msg=msg+"SCML Phase Residue: Mean = %7.3f\tSigma = %7.3f\tMax = %7.3f\tMin = %7.3f\n" % (mean,sigma,max_scmlp,min_scmlp) 
				
				mean=sumdalt/len(dalt)
				sigma=math.sqrt(sum2dalt/len(dalt)-mean*mean)
				ret["dalt"]=[mean, sigma, max_dalt, min_dalt]
				msg=msg+"Tilt  angle change: Mean = %7.3f\tSigma = %7.3f\tMax = %7.3f\tMin = %7.3f\n" % (mean,sigma,max_dalt,min_dalt) 
				
				mean=sumdaz/len(daz)
				sigma=math.sqrt(sum2daz/len(daz)-mean*mean)
				ret["daz"]=[mean, sigma, max_daz, min_daz]
				msg=msg+"Azimu angle change: Mean = %7.3f\tSigma = %7.3f\tMax = %7.3f\tMin = %7.3f\n" % (mean,sigma,max_daz,min_daz) 
				
				mean=sumdphi/len(dphi)
				sigma=math.sqrt(sum2dphi/len(dphi)-mean*mean)
				ret["dphi"]=[mean, sigma, max_dphi, min_dphi]
				msg=msg+"Phi   angle change: Mean = %7.3f\tSigma = %7.3f\tMax = %7.3f\tMin = %7.3f\n" % (mean,sigma,max_dphi,min_dphi)
				 
				mean=sumdx/len(dx)
				sigma=math.sqrt(sum2dx/len(dx)-mean*mean)
				ret["dx"]=[mean, sigma, max_dx, min_dx]
				msg=msg+"Center  x   change: Mean = %7.3f\tSigma = %7.3f\tMax = %7.3f\tMin = %7.3f\n" % (mean,sigma,max_dx,min_dx) 
				
				mean=sumdy/len(dy)
				sigma=math.sqrt(sum2dy/len(dy)-mean*mean)
				ret["dz"]=[mean, sigma, max_dy, min_dy]
				msg=msg+"Center  y   change: Mean = %7.3f\tSigma = %7.3f\tMax = %7.3f\tMin = %7.3f\n" % (mean,sigma,max_dy,min_dy)

				if self.Verbose: print msg
				ret["msg"]=msg
				return ret 
			except KeyboardInterrupt:
				return None
		else:
			return None
if __name__ == '__main__':
	print "%s: is designed as a library, but not as a standalone program" % sys.argv[0]
	sys.exit(1)
